from ..analysis import Analysis
from ..errors import AngrAnalysisError
from ..tablespecs import StringSpec
import logging
import simuvex
import re
import struct
from angr.path import Path

l = logging.getLogger("analysis.sleak")

class SleakError(AngrAnalysisError):
    pass

class SleakMeta(Analysis):
    """
    Stack leak detection - general stuff.
    See XSleak and Sleakslice for actual implementations.
    """

    out_functions = ['send', 'printf', 'vprintf', 'fprintf', 'vfprintf',
                 'wprintf', 'fwprintf', 'vwprintf', 'vfwprintf',
                 'putc', 'puts', 'putw', 'fputwc', 'putwc',
                 'putchar', 'fwrite', 'putc_unlocked',
                 'putchar_unlocked', 'writev', 'pwritev', 'pwritev64',
                 'pwrite', 'pwrite64', 'fwrite_unlocked', 'write']



    def __init__():
        raise Exception("Not implemented - use subclasses")


    """
    Methods subclasses must implement
    """

    def terminated_paths(self):
        """
        Returns a list of paths where the analysis stopped for whatever reasons
        """
        raise Exception("Not implemented - use subclasses")


    """
    General methods
    """


    def prepare(self, mode="all", istate=None, argc=2):
        """
        Explore the binary until targets are found.
        @targets: a tuple of manually identified targets.
        If @targets is none, we try to identify targets automatically.
        @mode:
            - "stack": track stack addresses.
            - "got": make the content of GOT stubs symbolic. It has the effect
               of tracking leakage of PLT stub addresses.
            - "data": TODO
            - "heap": track heap pointers
            - "none": track nothing (fast, just executes the slice)

        @argc: how many symbolic arguments to we want ?
        By default, we consider argv[1] as the filename, and argv[2] as being symbolic

        """

        # Find targets automatically
        self._targets = self.find_targets()
        if len(self._targets) == 0:
            l.warning("Could not find any target. Specify it manually (add_target()")

        try:
            self._malloc = self._p.loader.main_bin.get_call_stub_addr("malloc")
        except:
            self._malloc = None

        self._custom_targets={}
        self._custom_protos={}

        self.reached_target = False # Whether we made it to at least one target
        self.leaks = [] # Found leaking paths
        self.mode = mode if mode is not None else "all"

        # Stack
        self.stack_bottom = self._p.arch.initial_sp
        l.debug("Stack bottom is at 0x%x" % self.stack_bottom)
        self.stack_top = None
        self.tracked_addrs = []

        # Initial state
        if istate is None:
            istate = self._p.factory.entry_state(args = self._make_sym_argv(argc))

        # Initial path, comprising the initial state
        self.ipath = Path(self._p, istate)

        # Mode
        if "heap" in self.mode:
            self._set_heap_bp()

        elif "got" in self.mode:
            self.make_got_symbolic(self.ipath.state)

        elif "data" in self.mode:
            self._set_data_bp()

        elif "stack" in self.mode:
            self._set_stack_bp()

        elif "addr" in self.mode:
            self._set_addr_bp()

        elif "all" in self.mode:
            self._set_heap_bp()
            self._set_data_bp()
            self._set_addr_bp()
            self.make_got_symbolic(self.ipath.state)
            self._set_stack_bp()

        elif "none" in self.mode:
            l.info("Mode is none, won't track anything")
            pass # No tracking

        else:
            raise AngrAnalysisError("Invalid mode")

    def add_target(self, name, addr, proto):
        """
        Manually add a target to find.
        @proto is SleakProcedure prototype
        """
        self._custom_targets[name] = addr
        self._custom_protos[name] = proto

    def remove_target(self, name):
        try:
            self._targets.pop(name)
            self._custom_targets.pop(name)
        except:
            pass

    def set_malloc(self, addr):
        """
        Manually set the address of malloc() to track heap pointers
        """
        self._malloc = addr
        self._set_heap_bp()

    @property
    def targets(self):
        return dict(self._targets.items() + self._custom_targets.items())

    def _make_sym_argv(self, argc):
        """
        Make a symbolic argv, where argv[1] is concrete
        """
        if argc < 1:
            return []

        argv= [self._p.loader.main_bin.binary]

        if argc > 1:
            for i in range(1, argc):
                argv.append(StringSpec(sym_length=12))
        return argv

    ## Setting the breakpoints ##
    def _set_stack_bp(self):
        """
        Track stack pointer reads and make its content symbolic.
        """
        bp = simuvex.BP(simuvex.BP_BEFORE, action=self.make_sp_symbolic)
        self.ipath.state.inspect.add_breakpoint('reg_read', bp)

    def _set_data_bp(self):
        """
        Track access to data
        """
        self.make_data_symbolic(self.ipath.state)


    def _set_heap_bp(self):
        """
        Track malloc return pointers
        """

        if self._malloc is None:
            l.info("Could not find PLT stub addr for malloc, heap pointers "
                   "won't be tracked")
        else:
            action=self.make_heap_ptr_symbolic
            bp = simuvex.BP(simuvex.BP_AFTER, instruction=self._malloc, action=action)
            self.ipath.state.inspect.add_breakpoint('instruction', bp)
            l.info("Registering bp for malloc at 0x%x" % self._malloc)

    def _set_addr_bp(self):
        """
        Track anything that concretizes to an address inside the binary
        """

        # Mem writes: we store an expression in memory - this expression is an
        # address inside the binary
        self.ipath.state.inspect.add_breakpoint(
            'mem_write', simuvex.BP(simuvex.BP_AFTER, action=self.track_mem_write))

        #self.ipath.state.inspect.add_breakpoint(
        #    'reg_write', simuvex.BP(simuvex.BP_AFTER, action=self.track_reg_write))

    def find_targets(self):
        """
        What are the target addresses we are interested in ?
        These are output or interface functions.
        Returns a dict {name: addresses} where addresses are the PLT stubs of
        target functions.
        """
        targets={}
        for f in self.out_functions:
            if f in self._p.loader.main_bin.jmprel:
                try:
                    plt = self._p.loader.main_bin.get_call_stub_addr(f)
                    targets[f] = plt
                except:
                    l.warning("Could not detect plt stub addr for target %s" % repr(f))
                    pass

        l.info("Found targets (output functions) %s" % repr(targets))
        return targets
        #return tuple(targets.values())

    def _check_found_paths(self):
        """
        Iterates over all found paths to identify leaking ones
        """
        results = []
        if len(self.found_paths) > 0:
            self.reached_target = True

        # Found paths : output function reached
        for p in self.found_paths:
            r = self._check_path(p)
            if r is not None:
                results.append(r)
        self.leaks = results

    def _check_path(self, p):
        '''
        Check whether an individual path leaks
        '''
        sp = self.make_sleak_procedure(p)
        if sp is None:
            return
        if len(sp.badargs) > 0:
            l.info("Found leaking path - %s" % repr(sp))
            return sp

    def make_sleak_procedure(self, p):
        """
        @p: path where to make the SleakProcecure
        Note: the path must reach a known target, i.e., either one of the
        predefined ones, or the custom ones.
        """
        func = self._reached_target(p)
        if func is None:
            return
        sp = SleakProcedure(func, p, self.mode, self._find_proto(func))
        return sp

    def _find_proto(self, name):
        proto = []
        if name in self._custom_protos.keys():
            proto = self._custom_protos[name]
        return proto

    @property
    def found_paths(self):
        """
        Found paths: paths to the output functions
        """
        found=[]

        for p in self.terminated_paths:
            if self._reached_target(p) is not None:
                #found[p] = self._reached_target(p)
                found.append(p)

           # for succ in p.successors:
           #     if self._reached_target(succ) is not None:
           #         #found[succ] = self._reached_target(succ)
           #         found.append(succ)

        return found

    def _reached_target(self, p):
        """
        Which target was reached by path @p
        """
        for t in self.targets.values():
            if p.state.se.solution(p.addr,t):
                return self.target_name(t)

    """
    Args checking stuff
    """

    def target_name(self, addr):
        """
        Name from target addr
        """
        for name, target in self.targets.iteritems():
            if addr == target:
                return name


    """
    Stack tracking stuff
    """


    def track_mem_read(self, state):
        return self._track_mem_op(state, mode='r')

    def track_mem_write(self, state):
        addr_xpr = state.inspect.mem_write_expr
        caddr = self._check_is_mapped_addr(state, addr_xpr)
        if caddr is None:
            return

        mem_loc_xpr = state.inspect.mem_write_address
        mem_loc = state.se.any_int(mem_loc_xpr)

        l.info("Auto tracking addr 0x%x" % caddr)
        state.memory.make_symbolic("TRACKED_MAPPED_ADDR", mem_loc, self._p.arch.bits/8)
        #state.memory.make_symbolic("TRACKED_MAPPED_ADDR", caddr, self._p.arch.bits/8)
        #self.tracked_addrs.append({addr:state.memory.load(caddr, self._p.arch.bits/8)})

    def _check_is_mapped_addr(self, state, addr_xpr):
        """
        Check whether the given expr concretizes to an address inside the binary
        """
        addr = state.se.any_int(addr_xpr)
        try:
            caddr = self._canonical_addr(addr)
        except:
            return None

        if self._p.loader.addr_is_mapped(caddr):
            return caddr

    def track_reg_write(self, state):
        xpr = state.inspect.reg_write_expr
        caddr = self._check_is_mapped_addr(self, state, xpr)
        if caddr is None:
            return

        off_xpr = state.inspect.reg_write_offset
        off = state.se.any_int(off_xpr)

        l.info("Auto tracking addr 0x%x" % caddr)
        state.registers.make_symbolic("TRACKED_MAPPED_ADDR", off, self._p.arch.bits/8)
        #self.tracked_addrs.append({addr:state.memory.load(addr, self._p.arch.bits/8)})


    def _canonical_addr(self, addr):
        """
        Canonical representation of addr - unused
        """
        fmtin = self._p.arch.struct_fmt()
        if not "<" in fmtin:
            return addr

        fmtout = fmtin.replace('<','>')
        return struct.unpack(fmtout, struct.pack(fmtin, addr))[0]

    def _track_mem_op(self, state, mode=None):
        """
        Look for addresses in memory expressions
        """

        if mode == 'w':
            addr_xpr = state.inspect.mem_write_expr
            mem_loc_xpr = state.inspect.mem_write_address

        elif mode == 'r':
            addr_xpr = state.inspect.mem_read_expr
            mem_loc_xpr = state.inspect.mem_read_address
        else:
            raise Exception ("Invalid mode")

        '''
        A better, but expensive solution using the constraint solver:
        # Add a constraint and return a new expression
        nxpr = state.se.ULT(addr_xpr, 2^bits)
        if state.se.solution(nxpr, False):
            return
        if not state.se.unique(addr_xpr):
            l.debug("Warning: there are mu
        '''

        # This is faster, and acutally, addresses referring to objects of the
        # binary should be hardcoded as immediate values by the compiler anyway
        addr = state.se.any_int(addr_xpr)
        mem_loc = state.se.any_int(mem_loc_xpr)

        # Any address in the binary
        if self._p.loader.addr_is_mapped(addr):
            l.info("Auto tracking addr 0x%x [%s]" % (addr, mode))
            state.memory.make_symbolic("TRACKED_ADDR", mem_loc, self._p.arch.bits/8)
            self.tracked_addrs.append({addr:state.memory.load(addr, self._p.arch.bits/8)})


    def make_sp_symbolic(self, state):
        """
        Whatever we read from the stack pointer register is made symbolic.
        It has the effect of "tainting" all stack variable addresses (not their content).
        """
        if state.inspect.reg_write_offset == self._p.arch.sp_offset or state.inspect.reg_read_offset == self._p.arch.sp_offset:
            state.registers.make_symbolic("STACK_TRACK", self._p.arch.sp_offset, self._p.arch.bits/8)
            l.debug("SP set symbolic")

    def make_heap_ptr_symbolic(self, state):
        """
        Make heap pointer returned by malloc and cmalloc symbolic
        """
        #if state.inspect.function_name == "malloc" or state.inspect.function_name == "calloc":
        #convention = simuvex.Conventions[state.arch.name](state.arch)
        #addr = convention.return_val(state)
        reg = state.arch.ret_offset
        state.registers.make_symbolic("HEAP_TRACK", reg, self._p.arch.bits/8)
        #addr = state.se.any_int(state.registers.load(reg))
        #state.memory.make_symbolic("TRACKED_HEAPPTR", addr, self._p.arch.bits/8)
        #l.debug("Heap ptr @0x%x made symbolic" % state.se.any_int(addr))
        l.debug("Heap ptr made symbolic - reg off %d" % reg)

    def make_got_symbolic(self, state):
        """
        Make the content of GOT slots symbolic. The GOT addresses themselves
        are not made symbolic.
        """
        got = self._p.loader.main_bin.gotaddr
        gotsz = self._p.loader.main_bin.gotsz

        pltgot = self._p.loader.main_bin.pltgotaddr
        pltgotsz = self._p.loader.main_bin.pltgotsz

        jmprel = self._p.loader.main_bin.jmprel

        # First, let's try to do it using the dynamic info
        if len(jmprel) > 0:
            for symbol, addr in jmprel.iteritems():
                state.memory.make_symbolic("PLTGOT_TRACK", addr, self._p.arch.bits/8)
                l.debug("pltgot entry 0x%x (%s) made symbolic" % (addr, symbol))

        # Otherwise, we do it from static info, if there are any
        elif pltgotsz is not None:
            for addr in range(pltgot, pltgot+pltgotsz, self._p.arch.bits/8):
                state.memory.make_symbolic("PLTGOT_TRACK", addr, self._p.arch.bits/8)
                l.debug("Make PLTGOT addr 0x%x symbolic" % addr)

        else:
            l.info("We don't know where is got.plt in this binary, not GOT addr tracking")


        if gotsz is not None:
            for addr in range(got, got+gotsz, self._p.arch.bits/8):
                l.debug("Make GOT addr 0x%x symbolic" % addr)
                state.memory.make_symbolic("GOT_TRACK", addr, self._p.arch.bits/8)


    def make_data_symbolic(self, state):
        """
        This is broken
        """
        strtab = self._p.loader.main_bin.strtab_vaddr
        for off, _ in self._p.loader.main_bin.strtab.iteritems():
            addr = off + strtab
            state.memory.make_symbolic("DATA_TRACK", addr, self._p.arch.bits/8)

    def get_stack_top(self, state):
        """
        We keep tracks of the highest stack address the program has accessed.
        """

        # We suppose the stack pointer has only one concrete solution
        sp = state.se.any_int(state.registers.load(self._p.arch.sp_offset))

        if self.stack_top is None:
            self.stack_top = sp
        else:
           if sp < self.stack_top:
               self.stack_top = sp
        l.debug("Stack top is at 0x%x" % self.stack_top)

    def is_stack_addr(self, addr, state):
        self.get_stack_top(state)
        return addr >= self.stack_top and addr <= self.stack_bottom


class SleakProcedure(object):
    """
    SleakProcedure: check procedure parameters.
    It only interprets what the procedure outputs in terms of pointers.
    """

    # Parameters to functions expressed in terms of pointers (p) or values (v)
    _fn_parameters={}
    _fn_parameters['puts'] = ['p']
    _fn_parameters['send'] = ['v', 'p', 'v', 'v' ]
    _fn_parameters['printf'] = []
    _fn_parameters['fprintf'] = []
    _fn_parameters['vprintf'] = []
    _fn_parameters['vfprintf'] = []
    _fn_parameters['wprintf'] = []
    _fn_parameters['fwprintf'] = []
    _fn_parameters['vwprintf'] = []
    _fn_parameters['vfwprintf'] = []
    _fn_parameters['write'] = ['v', 'p', 'v']
    _fn_parameters['putc'] = ['v', 'p']
    _fn_parameters['puts'] = ['p']
    _fn_parameters['putw'] = ['v', 'p']
    _fn_parameters['putwc'] = ['v', 'p']
    _fn_parameters['fputwc'] = ['v', 'p']
    _fn_parameters['putchar'] = ['v']
    _fn_parameters['fwrite'] = ['p', 'v', 'v', 'p']
    _fn_parameters['fwrite_unlocked'] = ['p', 'v', 'v', 'p']
    _fn_parameters['pwrite'] = ['v', 'p', 'v', 'v']
    _fn_parameters['putc_unlocked'] = ['v','p']
    _fn_parameters['putchar_unlocked'] = ['v']
    _fn_parameters['writev'] = ['v', 'p', 'v']
    _fn_parameters['pwritev'] = ['v', 'p', 'v', 'v']
    _fn_parameters['pwritev64'] = ['v', 'p', 'v', 'v']
    _fn_parameters['pwrite'] = ['v', 'p', 'v', 'v']

    def __init__(self, name, path, mode='track_sp', fn_parameters=[]):
        """
        @name: name of the function
        @path: path reaching this function (or its PLT stub)
        @fn_parameters: custom prototype for this function (see self._fn_parameters)
        """

        self.path = path
        self.state = self.path.state # expected: the initial state of the PLT stub
        self.name = name
        self.mode = mode

        # Add custom prototype to the list
        if len(fn_parameters) > 0:
            self._fn_parameters[name] = fn_parameters

        # Functions depending on a format string
        if len(self._fn_parameters[name]) == 0:
            # The first argument to a function that requires a format string is
            # a pointer to the format string itself.
            self.types = ['p'] + self._parse_format_string(self.get_format_string())
            l.debug("Got types vector %s" % repr(self.types))
            self.n_args = len(self.types) # The format string and the args
        else:
            self.types = self._fn_parameters[name]
            self.n_args = len(self.types)

        self.badargs = self._check_args() # args leaking pointer info

    def get_arg_expr(self, arg_num):
        """
        What is the expression of argument number @arg_num ?
        """
        convention = Conventions[self.state.arch.name](self.state.arch)
        return convention.peek_arg(arg_num, self.state)

    def get_arg_val(self, arg_num):
        expr = self.get_arg_expr(arg_num)
        if not self.state.se.unique(expr):
            l.warning("There are multiple solutions, this is just one")
        return self.state.se.any_int(expr)

    def _check_args(self):
        """
        Check whether any of the args contains information about a stack (or
        tracked) address.
        """

        count = self.n_args
        args={}
        for i in range(0, count):
            args[i] = self.get_arg_expr(i)

        matching={}
        for arg_num, expr in args.iteritems():
            if self._check_ptr_leak(expr, arg_num):
                matching[arg_num] = self.state.se.simplify(expr)

        return matching

    def _check_ptr_leak(self, expr, arg_num):
        """
        Check whether @expr passed as argument number @arg_num to the output
        function (self) ends up leaking address information.
        """
        # Type of the argument w.r.t the function's prototype
        arg_type = self.types[arg_num]

        # Does expr depends on a stack_addr ?
        if self._arg_depends_on_address(expr):
            # Pointer (or variable depending on pointer) passed as value, that's
            # a leak !
            if arg_type == 'v':
                return True

        # Otherwise, if we got a pointer for a pointer, nothing wrong... but the
        # output function is going to dereference it, and
        # the target of the pointer might, in turn, depend on an address ?
        if arg_type == 'p':
            if not self.state.se.unique(expr):
                #raise Exception("Oops, we got a symbolic pointer...")
                l.info("We got a symbolic pointer...")

            addr = self.state.se.any_int(expr)
            val = self.state.memory.load(addr, self.state.arch.bits/8)
            if self._arg_depends_on_address(val):
                return True
        return False

    def get_format_string(self):
        """
        Determines the number of arguments passed to printf-like functions based
        on the format string, given the state @state
        """
        # The address of the first argument (the pointer to the format string)
        arg0 = self.get_arg_expr(0)
        if not self.state.se.unique(arg0):
            raise Exception("Symbolic string pointer... something is probably wrong")

        addr = self.state.se.any_int(arg0)

        string = ""
        size = 0

        # We increaze the size of the string by 10 characters each time
        # until we find the ending \x00
        while len(re.findall("\x00", string)) == 0:
            size = size + 10
            string = self.state.se.any_str(self.state.memory.load(addr, size))

        # Only get the part of the string we are interested in
        return string[0:string.find('\x00')]

    def _parse_format_string(self, fstr):

        '''
        TODO: We should assume that a format string:
            - starts with %
            - contains a number of characters for precision, length etc
            - ends with one of the specifiers cduxXefg%
        '''
        #fmt = re.findall(r'%.*(?cduxXefg%', fstr)

        fmt = re.findall(r'%[a-z]+', fstr)
        return map(self._format_str_types, fmt)

    def _format_str_types(self, fmt):
        """
        Conversion of format str types to simple types 'v' or 'p'
        """
        if fmt == "%s": #or fmt == "%p":
            return "p"
        else:
            return "v"

    def _arg_depends_on_address(self, arg_expr):
        """
        This determines whether the argument depends on an address (tracked or stack)
        """

        tstr = "TRACK"

        if tstr in repr(arg_expr):
            return True
        return False

    def __repr__(self):
        addr = self.path.addr
        return "<SleakProcedure at addr 0x%x (%s)>" % (addr, self.name)


"""
Calling conventions for supported architectures.
"""
class Convention(object):
    def __init__(self, arch):
        self.arch = arch
        self.skip=0

    def call_convention(self):
        call = self._call_convention()
        if call is None:
            raise Exception("Unsupported architecture for this convention")
        return call

    def _call_convention(self):
        raise Exception("Not yet implemented !")

    def return_addr(self):
        raise Exception("Not yet implemented !")

    def return_val(self, state):
        off = self.arch.ret_offset
        return state.registers.load(off, endness = state.arch.register_endness)

    def peek_arg(self, index, state):
        """
        Given a state, peek arg number @index from the right place (stack,
        registers...)
        """
        # Skip the return address
        args_mem_base = state.registers.load(state.arch.sp_offset) + self.arch.bits/8
        return self._arg_getter(self.call_convention(), args_mem_base,
                               self.arch.bits/8, index, state)

    def _arg_getter(self, reg_offsets, args_mem_base, stack_step, index,
                    state):
            """
            This function does NOT add refs, be careful to wrap it into something
            that does when it matters
            """
            if index < len(reg_offsets):
                expr = state.registers.load(reg_offsets[index],
                                           endness=state.arch.register_endness)
            else:
                index -= len(reg_offsets)
                mem_addr = args_mem_base + (index * stack_step)
                expr = state.memory.load(mem_addr, stack_step,
                                           endness=state.arch.memory_endness)
            return expr

class SYSCALL(Convention):
    def _call_convention(self):
        if self.arch.name == "AMD64":
            # Reg offsets for rdi, rsi, rdx, rcx, r8, r9
            return [ 72, 64, 32, 24, 80, 88 ]

class CDECL(Convention):

    def _call_convention(self):
        if self.arch.name== "X86":
            return [] # all on the stack

"""
Architecture specific stuff
"""

class Systemv_x64(Convention):
    def _call_convention(self):
            if self.arch.name == "AMD64":
                # rdi, rsi, rdx, rcx, r8, r9
                return [ 72, 64, 32, 24, 80, 88 ]

class ARM(Convention):
    def _call_convention(self):
        if self.arch.name in ("ARMEL", "ARMHF"):
            # Reg offsets of r0, r1, r2, r3
            return [ 8, 12, 16, 20 ]

class PPC32(Convention):
    def _call_convention(self):
        if self.arch.name == "PPC32":
            return [ 28, 32, 36, 40, 44, 48, 52, 56 ] # r3 through r10

class PPC64(Convention):
    def _call_convention(self):
        if self.arch.name == "PPC64":
            return [ 40, 48, 56, 64, 72, 80, 88, 96 ] # r3 through r10

class MIPS32(Convention):
    def _call_convention(self):
        if self.arch.name == "MIPS32":
            return [ 'a0', 'a1', 'a2', 'a3' ] # r4 through r7

Conventions = {'AMD64': Systemv_x64,
                        'ARM': ARM,
                        'PPC32': PPC32,
                        'PPC64': PPC64,
                        'MIPS32': MIPS32,
                        'X86': CDECL,
                        'SYSCALL': SYSCALL
                        }



import logging
import string
import math
import re
import os
import pickle
from datetime import datetime
from collections import defaultdict

import networkx
import progressbar

import simuvex
import cle
import pyvex

from ..errors import AngrError
from ..analysis import Analysis
from ..surveyors import Explorer, Slicecutor
from ..annocfg import AnnotatedCFG

l = logging.getLogger("angr.analyses.girlscout")

class SegmentList(object):
    def __init__(self):
        self._list = []
        self._bytes_occupied = 0

    def _search(self, addr):
        start = 0
        end = len(self._list)
        mid = 0
        while start != end:
            mid = (start + end) / 2
            # print ">start = %d, end = %d, mid = %d" % (start, end, mid)
            tpl = self._list[mid]
            if addr < tpl[0]:
                end = mid
            elif addr >= tpl[1]:
                start = mid + 1
            else:
                # Overlapped :(
                start = mid
                break
            # print "<start = %d, end = %d, mid = %d" % (start, end, mid)
        return start

    def _merge_check(self, address, size, idx):
        # Shall we merge it with the one in front?
        merged = False
        new_start = address
        new_end = address + size
        new_idx = idx
        while new_idx > 0:
            new_idx -= 1
            tpl = self._list[new_idx]
            if new_start <= tpl[1]:
                del self._list[new_idx]
                new_start = min(tpl[0], address)
                new_end = max(tpl[1], address + size)
                self._list.insert(new_idx,
                                  (new_start, new_end))
                self._bytes_occupied += (new_end - new_start) - (tpl[1] - tpl[0])
                merged = True
            else:
                break

        if not merged:
            if idx == len(self._list):
                self._list.append((address, address + size))
            else:
                self._list.insert(idx, (address, address + size))
            self._bytes_occupied += size

    def has_blocks(self):
        return len(self._list) > 0

    def next_free_pos(self, address):
        idx = self._search(address + 1)
        if idx < len(self._list) and \
                address >= self._list[idx][0] and address < self._list[idx][1]:
            # Occupied
            i = idx
            while i + 1 < len(self._list) and \
                    self._list[i][1] == self._list[i + 1][0]:
                i += 1
            if i == len(self._list):
                return self._list[-1][1]
            else:
                return self._list[i][1]
        else:
            return address + 1

    def is_occupied(self, address):
        idx = self._search(address)
        if address >= self._list[idx][0] and address <= self._list[idx][1]:
            return True
        if idx > 0 and \
                address <= self._list[idx - 1][1]:
            return True
        return False

    def occupy(self, address, size):
        # l.debug("Occpuying 0x%08x-0x%08x", address, address + size)
        if len(self._list) == 0:
            self._list.append((address, address + size))
            self._bytes_occupied += size
            return
        # Find adjacent element in our list
        idx = self._search(address)
        # print idx
        if idx == len(self._list):
            # We should append at the end
            self._merge_check(address, size, idx)
        else:
            tpl = self._list[idx]
            # Overlap check
            if address <= tpl[0] and address + size >= tpl[0] or \
                    address <= tpl[1] and address + size >= tpl[1] or \
                    address >= tpl[0] and address + size <= tpl[1]:
                new_start = min(address, tpl[0])
                new_end = max(address + size, tpl[1])
                self._list[idx] = (new_start, new_end)
                # Shall we merge it with the previous one?
                # Remember to remove this one if we shall merge it with the one
                # in front!
                while idx > 0:
                    idx -= 1
                    if new_start <= self._list[idx][1]:
                        new_start = min(self._list[idx][0], new_start)
                        new_end = max(self._list[idx][1], new_end)
                        self._list[idx] = (new_start, new_end)
                        del self._list[idx + 1]
                    else:
                        break
            else:
                # It's not overlapped with this one
                # Shall we merge it with the previous one?
                self._merge_check(address, size, idx)
        # l.debug(self._dbg_output())
        # self._debug_check()

    @property
    def occupied_size(self):
        return self._bytes_occupied

    def _dbg_output(self):
        s = "["
        lst = []
        for start, end in self._list:
            lst.append("(0x%08x, 0x%08x)" % (start, end))
        s += ", ".join(lst)
        s += "]"
        return s

    def _debug_check(self):
        old_start = 0
        old_end = 0
        for start, end in self._list:
            if start <= old_end:
                raise Exception("Error in SegmentList: blocks are not merged")
            old_start = start
            old_end = end

class GirlScout(Analysis):
    '''
    We find functions inside the given binary, try to decide the base address if needed, and build a control-flow
    graph on top of that to see if there is an entry or not. Obviously if the binary is not loaded as a blob (not
    using Blob as its backend), GirlScout will not try to determine the base address.

    It's also optional to perform a full code scan of the binary to show where all codes are. By default we don't scan
    the entire binary since it's time consuming.

    You probably need a BoyScout to determine the possible architecture and endianess of your binary blob.
    '''

    def __init__(self, binary=None, start=None, end=None, pickle_intermediate_results=False, perform_full_code_scan=False):
        self._project = self._p
        self._binary = binary if binary is not None else self._p.loader.main_bin
        self._start = start if start is not None else (self._binary.rebase_addr + self._binary.get_min_addr())
        self._end = end if end is not None else (self._binary.rebase_addr + self._binary.get_max_addr())
        self._pickle_intermediate_results = pickle_intermediate_results
        self._perform_full_code_scan = perform_full_code_scan

        l.debug("Starts at 0x%08x and ends at 0x%08x.", self._start, self._end)

        # Valid memory regions
        self._valid_memory_regions = sorted(
            [ (self._binary.rebase_addr+start, self._binary.rebase_addr+start+len(cbacker))
                for start, cbacker in self._project.loader.memory.cbackers ],
            key=lambda x: x[0]
        )
        self._valid_memory_region_size = sum([ (end - start) for start, end in self._valid_memory_regions ])

        # Size of each basic block
        self._block_size = { }

        self._next_addr = self._start - 1
        # Starting point of functions
        self._functions = None
        # Calls between functions
        self._call_map = networkx.DiGraph()
        # A CFG - this is not what you get from project.analyses.CFG() !
        self._cfg = networkx.DiGraph()
        # Create the segment list
        self._seg_list = SegmentList()

        self._read_addr_to_run = defaultdict(list)
        self._write_addr_to_run = defaultdict(list)

        # All IRSBs with an indirect exit target
        self._indirect_jumps = set()

        self._unassured_functions = set()

        self._base_address = None

        self.result = { }

        # Start working!
        self._reconnoiter()

        # Set all results
        self.result['base_address'] = self._base_address
        self.result['call_map'] = self._call_map
        self.result['cfg'] = self._cfg
        self.result['functions'] = self._functions

    @property
    def call_map(self):
        return self._call_map

    def _get_next_addr_to_search(self, alignment=None):
        # TODO: Take care of those functions that are already generated
        curr_addr = self._next_addr
        if self._seg_list.has_blocks:
            curr_addr = self._seg_list.next_free_pos(curr_addr)

        if alignment is not None:
            if curr_addr % alignment > 0:
                curr_addr = curr_addr - curr_addr % alignment + alignment

        # Make sure curr_addr exists in binary
        accepted = False
        for start, end in self._valid_memory_regions:
            if curr_addr >= start and curr_addr < end:
                # accept
                accepted = True
                break
            if curr_addr < start:
                # accept, but we are skipping the gap
                accepted = True
                curr_addr = start

        if not accepted:
            # No memory available!
            return None

        self._next_addr = curr_addr
        if self._end is None or curr_addr < self._end:
            l.debug("Returning new recon address: 0x%08x", curr_addr)
            return curr_addr
        else:
            l.debug("0x%08x is beyond the ending point.", curr_addr)
            return None

    def _get_next_code_addr(self, initial_state):
        '''
        Besides calling _get_next_addr, we will check if data locates at that
        address seems to be code or not. If not, we'll move on to request for
        next valid address.
        '''
        next_addr = self._get_next_addr_to_search()
        if next_addr is None:
            return None

        start_addr = next_addr
        sz = ""
        is_sz = True
        while is_sz:
            # Get data until we meet a 0
            while next_addr in initial_state.memory:
                try:
                    l.debug("Searching address %x", next_addr)
                    val = initial_state.mem_concrete(next_addr, 1)
                    if val == 0:
                        if len(sz) < 4:
                            is_sz = False
                        else:
                            reach_end = True
                        break
                    if chr(val) not in string.printable:
                        is_sz = False
                        break
                    sz += chr(val)
                    next_addr += 1
                except simuvex.SimValueError:
                    # Not concretizable
                    l.debug("Address 0x%08x is not concretizable!", next_addr)
                    break

            if len(sz) > 0 and is_sz:
                l.debug("Got a string of %d chars: [%s]", len(sz), sz)
                # l.debug("Occpuy %x - %x", start_addr, start_addr + len(sz) + 1)
                self._seg_list.occupy(start_addr, len(sz) + 1)
                sz = ""
                next_addr = self._get_next_addr_to_search()
                if next_addr is None:
                    return None
                # l.debug("next addr = %x", next_addr)
                start_addr = next_addr

            if is_sz:
                next_addr += 1

        instr_alignment = initial_state.arch.instruction_alignment
        if start_addr % instr_alignment > 0:
            start_addr = start_addr - start_addr % instr_alignment + \
                instr_alignment

        l.debug('_get_next_code_addr() returns 0x%x', start_addr)
        return start_addr

    def _symbolic_reconnoiter(self, addr, target_addr, max_depth=10):
        '''
        When an IRSB has more than two exits (for example, a jumptable), we
        cannot concretize their exits in concrete mode. Hence we statically
        execute the function from beginning in this method, and then switch to
        symbolic mode for the final IRSB to get all possible exits of that
        IRSB.
        '''
        state = self._project.factory.blank_state(addr=addr,
                                                  mode="symbolic",
                                                  add_options={simuvex.o.CALLLESS}
                                                 )
        initial_exit = self._project.factory.path(state)
        explorer = Explorer(self._project,
                            start=initial_exit,
                            max_depth=max_depth,
                            find=(target_addr), num_find=1).run()
        if len(explorer.found) > 0:
            path = explorer.found[0]
            last_run = path.last_run
            return last_run.flat_exits()
        else:
            return []

    def _static_memory_slice(self, run):
        if isinstance(run, simuvex.SimIRSB):
            for stmt in run.statements:
                refs = stmt.actions
                if len(refs) > 0:
                    real_ref = refs[-1]
                    if type(real_ref) == simuvex.SimActionData:
                        if real_ref.action == 'write':
                            addr = real_ref.addr
                            if not run.initial_state.se.symbolic(addr):
                                concrete_addr = run.initial_state.se.any_int(addr)
                                self._write_addr_to_run[addr].append(run.addr)
                        elif real_ref.action == 'read':
                            addr = real_ref.addr
                            if not run.initial_state.se.symbolic(addr):
                                concrete_addr = run.initial_state.se.any_int(addr)
                            self._read_addr_to_run[addr].append(run.addr)

    def _scan_code(self, traced_addresses, function_exits, initial_state, starting_address):
        # Saving tuples like (current_function_addr, next_exit_addr)
        # Current_function_addr == -1 for exits not inside any function
        remaining_exits = set()
        next_addr = starting_address

        # Initialize the remaining_exits set
        remaining_exits.add((next_addr,
                             next_addr,
                             next_addr,
                             initial_state.copy()))

        while len(remaining_exits):
            current_function_addr, previous_addr, parent_addr, state = \
                remaining_exits.pop()
            if previous_addr in traced_addresses:
                continue

            # Add this node to the CFG first, in case this is a dangling node
            self._cfg.add_node(previous_addr)

            if current_function_addr != -1:
                l.debug("Tracing new exit 0x%08x in function 0x%08x",
                        previous_addr, current_function_addr)
            else:
                l.debug("Tracing new exit 0x%08x", previous_addr)
            traced_addresses.add(previous_addr)

            self._scan_block(previous_addr, state, current_function_addr, function_exits, remaining_exits, traced_addresses)


    def _scan_block(self, addr, state, current_function_addr, function_exits, remaining_exits, traced_addresses):
        # Let's try to create the pyvex IRSB directly, since it's much faster

        try:
            irsb = self._project.factory.block(addr).vex

            # Log the size of this basic block
            self._block_size[addr] = irsb.size

            # Occupy the block
            self._seg_list.occupy(addr, irsb.size)
        except (AngrTranslationError, AngrMemoryError):
            return

        # Get all possible successors
        next, jumpkind = irsb.next, irsb.jumpkind
        successors = [ (i.dst, i.jumpkind) for i in irsb.statements if type(i) is pyvex.IRStmt.Exit]
        successors.append((next, jumpkind))

        # Process each successor
        for suc in successors:
            target, jumpkind = suc

            if type(target) is pyvex.IRExpr.Const:
                next_addr = target.con.value
            else:
                next_addr = None

            if jumpkind == 'Ijk_Boring' and next_addr is not None:
                remaining_exits.add((current_function_addr, next_addr,
                                     addr, None))

            elif jumpkind == 'Ijk_Call' and next_addr is not None:
                # Log it before we cut the tracing :)
                if jumpkind == "Ijk_Call":
                    if current_function_addr != -1:
                        self._functions.add(current_function_addr)
                        self._functions.add(next_addr)
                        self._call_map.add_edge(current_function_addr, next_addr)
                    else:
                        self._functions.add(next_addr)
                        self._call_map.add_node(next_addr)
                elif jumpkind == "Ijk_Boring" or \
                                jumpkind == "Ijk_Ret":
                    if current_function_addr != -1:
                        function_exits[current_function_addr].add(next_addr)

                # If we have traced it before, don't trace it anymore
                if next_addr in traced_addresses:
                    return

                remaining_exits.add((next_addr, next_addr, addr, None))
                l.debug("Function calls: %d", len(self._call_map.nodes()))

    def _scan_block_(self, addr, state, current_function_addr, function_exits, remaining_exits, traced_addresses):

        # Get a basic block
        state.ip = addr

        s_path = self._project.factory.path(state)
        try:
            s_run = s_path.next_run
        except simuvex.SimIRSBError, ex:
            l.debug(ex)
            return
        except AngrError, ex:
            # "No memory at xxx"
            l.debug(ex)
            return
        except (simuvex.SimValueError, simuvex.SimSolverModeError), ex:
            # Cannot concretize something when executing the SimRun
            l.debug(ex)
            return
        except simuvex.SimError as ex:
            # Catch all simuvex errors
            l.debug(ex)
            return

        if type(s_run) is simuvex.SimIRSB:
            # Calculate its entropy to avoid jumping into uninitialized/all-zero space
            bytes = s_run.irsb._state[1]['bytes']
            size = s_run.irsb.size
            ent = self._calc_entropy(bytes, size=size)
            if ent < 1.0 and size > 40:
                # Skipping basic blocks that have a very low entropy
                return

        # self._static_memory_slice(s_run)

        # Mark that part as occupied
        if isinstance(s_run, simuvex.SimIRSB):
            self._seg_list.occupy(addr, s_run.irsb.size)
        successors = s_run.flat_successors + s_run.unsat_successors
        has_call_exit = False
        tmp_exit_set = set()
        for suc in successors:
            if suc.scratch.jumpkind == "Ijk_Call":
                has_call_exit = True

        for suc in successors:
            jumpkind = suc.scratch.jumpkind

            if has_call_exit and jumpkind == "Ijk_Ret":
                jumpkind = "Ijk_FakeRet"

            if jumpkind == "Ijk_Ret":
                continue

            try:
                # Try to concretize the target. If we can't, just move on
                # to the next target
                next_addr = suc.se.exactly_n_int(suc.ip, 1)[0]
            except (simuvex.SimValueError, simuvex.SimSolverModeError) as ex:
                # Undecidable jumps (might be a function return, or a conditional branch, etc.)

                # We log it
                self._indirect_jumps.add((suc.scratch.jumpkind, addr))
                l.info("IRSB 0x%x has an indirect exit %s.", addr, suc.scratch.jumpkind)

                continue

            self._cfg.add_edge(addr, next_addr, jumpkind=jumpkind)
            # Log it before we cut the tracing :)
            if jumpkind == "Ijk_Call":
                if current_function_addr != -1:
                    self._call_map.add_edge(current_function_addr, next_addr)
                else:
                    self._call_map.add_node(next_addr)
            elif jumpkind == "Ijk_Boring" or \
                            jumpkind == "Ijk_Ret":
                if current_function_addr != -1:
                    function_exits[current_function_addr].add(next_addr)

            # If we have traced it before, don't trace it anymore
            if next_addr in traced_addresses:
                continue
            # If we have traced it in current loop, don't tract it either
            if next_addr in tmp_exit_set:
                continue

            tmp_exit_set.add(next_addr)

            if jumpkind == "Ijk_Call":
                # This is a call. Let's record it
                new_state = suc.copy()
                # Unconstrain those parameters
                # TODO: Support other archs as well
                # if 12 + 16 in new_state.registers.mem:
                #    del new_state.registers.mem[12 + 16]
                #if 16 + 16 in new_state.registers.mem:
                #    del new_state.registers.mem[16 + 16]
                #if 20 + 16 in new_state.registers.mem:
                #    del new_state.registers.mem[20 + 16]
                # 0x8000000: call 0x8000045
                remaining_exits.add((next_addr, next_addr, addr, new_state))
                l.debug("Function calls: %d", len(self._call_map.nodes()))
            elif jumpkind == "Ijk_Boring" or \
                            jumpkind == "Ijk_Ret" or \
                            jumpkind == "Ijk_FakeRet":
                new_state = suc.copy()
                l.debug("New exit with jumpkind %s", jumpkind)
                # FIXME: should not use current_function_addr if jumpkind is "Ijk_Ret"
                remaining_exits.add((current_function_addr, next_addr,
                                     addr, new_state))
            elif jumpkind == "Ijk_NoDecode":
                # That's something VEX cannot decode!
                # We assume we ran into a deadend
                pass
            elif jumpkind.startswith("Ijk_Sig"):
                # Should not go into that exit
                pass
            elif jumpkind == "Ijk_TInval":
                # ppc32: isync
                # FIXME: It is the same as Ijk_Boring! Process it later
                pass
            elif jumpkind == 'Ijk_Sys_syscall':
                # Let's not jump into syscalls
                pass
            elif jumpkind == 'Ijk_InvalICache':
                pass
            elif jumpkind == 'Ijk_MapFail':
                pass
            elif jumpkind == 'Ijk_EmWarn':
                pass
            else:
                raise Exception("NotImplemented")

    def _scan_function_prologues(self, traced_address, function_exits, initial_state):
        '''
        Scan the entire program space for prologues, and start code scanning at those positions
        :param traced_address:
        :param function_exits:
        :param initial_state:
        :param next_addr:
        :return:
        '''

        # Precompile all regexes
        regexes = set()
        for ins_regex in self._p.arch.function_prologs:
            r = re.compile(ins_regex)
            regexes.add(r)

        # TODO: Make sure self._start is aligned

        # Construct the binary blob first
        # TODO: We shouldn't directly access the _memory of main_bin. An interface
        # to that would be awesome.

        strides = self._p.loader.main_bin.memory.stride_repr

        for start_, end_, bytes in strides:
            for regex in regexes:
                # Match them!
                for mo in regex.finditer(bytes):
                    position = mo.start() + start_
                    if position % self._p.arch.instruction_alignment == 0:
                        if position not in traced_address:
                            percentage = self._seg_list.occupied_size * 100.0 / (self._valid_memory_region_size)
                            l.info("Scanning %xh, progress %0.04f%%", position, percentage)

                            self._unassured_functions.add(position)

                            self._scan_code(traced_address, function_exits, initial_state, position)
                        else:
                            l.info("Skipping %xh", position)

    def _process_indirect_jumps(self):
        '''
        Execute each basic block with an indeterminiable exit target
        :return:
        '''

        function_starts = set()
        print "We have %d indirect jumps" % len(self._indirect_jumps)

        for jumpkind, irsb_addr in self._indirect_jumps:
            # First execute the current IRSB in concrete mode

            if len(function_starts) > 20:
                break

            if jumpkind == "Ijk_Call":
                state = self._p.factory.blank_state(addr=irsb_addr, mode="concrete",
                                                    add_options={simuvex.o.SYMBOLIC_INITIAL_VALUES}
                                                   )
                path = self._p.factory.path(state)
                print hex(irsb_addr)

                try:
                    r = (path.next_run.successors + path.next_run.unsat_successors)[0]
                    ip = r.se.exactly_n_int(r.ip, 1)[0]

                    function_starts.add(ip)
                    continue
                except simuvex.SimSolverModeError as ex:
                    pass

                # Not resolved
                # Do a backward slicing from the call
                irsb = self._p.factory.block(irsb_addr).vex
                stmts = irsb.statements
                # Start slicing from the "next"

                b = Blade(self._cfg, irsb.addr, -1, project=self._p)

                # Debugging output
                for addr, stmt_idx in sorted(list(b.slice.nodes())):
                    irsb = self._p.factory.block(addr).vex
                    stmts = irsb.statements
                    print "%x: %d | " % (addr, stmt_idx),
                    print "%s" % stmts[stmt_idx],
                    print "%d" % b.slice.in_degree((addr, stmt_idx))

                print ""

                # Get all sources
                sources = [n for n in b.slice.nodes() if b.slice.in_degree(n) == 0]

                # Create the annotated CFG
                annotated_cfg = AnnotatedCFG(self._p, None, target_irsb_addr=irsb_addr, detect_loops=False)
                annotated_cfg.from_digraph(b.slice)

                for src_irsb, src_stmt_idx in sources:
                    # Use slicecutor to execute each one, and get the address
                    # We simply give up if any exception occurs on the way

                    start_state = self._p.factory.blank_state(addr=src_irsb,
                                                              add_options=
                                                              {simuvex.o.DO_RET_EMULATION,
                                                               simuvex.o.TRUE_RET_EMULATION_GUARD}
                                                             )

                    start_path = self._p.factory.path(start_state)

                    # Create the slicecutor
                    slicecutor = Slicecutor(self._p, annotated_cfg, start=start_path, targets=(irsb_addr,))

                    # Run it!
                    try:
                        slicecutor.run()
                    except KeyError as ex:
                        # This is because the program slice is incomplete.
                        # Blade will support more IRExprs and IRStmts
                        l.debug("KeyError occurred due to incomplete program slice.", exc_info=ex)
                        continue

                    # Get the jumping targets
                    for r in slicecutor.reached_targets:
                        if r.next_run.successors:
                            target_ip = r.next_run.successors[0].ip
                            se = r.next_run.successors[0].se

                            if not se.symbolic(target_ip):
                                concrete_ip = se.exactly_n_int(target_ip, 1)[0]
                                function_starts.add(concrete_ip)
                                l.info("Found a function address %x", concrete_ip)

        return function_starts

    def _solve_for_base_address(self, function_starts, functions):
        '''
        Voting for the most possible base address.

        :param function_starts:
        :param functions:
        :return:
        '''

        pseudo_base_addr = self._p.loader.main_bin.get_min_addr()

        base_addr_ctr = { }

        for s in function_starts:
            for f in functions:
                base_addr = s - f + pseudo_base_addr
                ctr = 1

                for k in function_starts:
                    if k - base_addr + pseudo_base_addr in functions:
                        ctr += 1

                if ctr > 5:
                    base_addr_ctr[base_addr] = ctr

        if len(base_addr_ctr):
            base_addr, hits = sorted([(k, v) for k, v in base_addr_ctr.iteritems()], key=lambda x: x[1], reverse=True)[0]

            return base_addr
        else:
            return None


    def _reconnoiter(self):

        if type(self._binary) is cle.blob.Blob:
            self._determine_base_address()

        if self._perform_full_code_scan:
            self._full_code_scan()

    def _determine_base_address(self):
        '''
        The basic idea is simple: start from a specific point, try to construct
        functions as much as we can, and maintain a function distribution graph
        and a call graph simultaneously. Repeat searching until we come to the
        end that there is no new function to be found.
        A function should start with:
            # some addresses that a call exit leads to, or
            # certain instructions. They are recoreded in SimArch.

        For a better performance, instead of blindly scanning the entire process
        space, we first try to search for instruction patterns that a function
        may start with, and start scanning at those positions. Then we try to
        decode anything that is left.
        '''

        traced_address = set()
        self._functions = set()
        self._call_map = networkx.DiGraph()
        self._cfg = networkx.DiGraph()
        initial_state = self._project.factory.blank_state(mode="fastpath")
        initial_options = initial_state.options - { simuvex.o.TRACK_CONSTRAINTS } - simuvex.o.refs
        initial_options |= { simuvex.o.SUPER_FASTPATH }
        # initial_options.remove(simuvex.o.COW_STATES)
        initial_state.options = initial_options
        # Sadly, not all calls to functions are explicitly made by call
        # instruction - they could be a jmp or b, or something else. So we
        # should record all exits from a single function, and then add
        # necessary calling edges in our call map during the post-processing
        # phase.
        function_exits = defaultdict(set)

        dump_file_prefix = self._p.filename

        if self._pickle_intermediate_results and \
                os.path.exists(dump_file_prefix + "_indirect_jumps.angr"):
            l.debug("Loading existing intermediate results.")
            self._indirect_jumps = pickle.load(open(dump_file_prefix + "_indirect_jumps.angr", "rb"))
            self._cfg = pickle.load(open(dump_file_prefix + "_coerce_cfg.angr", "rb"))
            self._unassured_functions = pickle.load(open(dump_file_prefix + "_unassured_functions.angr", "rb"))
        else:
            # Performance boost :-)
            # Scan for existing function prologues
            self._scan_function_prologues(traced_address, function_exits, initial_state)

            if self._pickle_intermediate_results:
                l.debug("Dumping intermediate results.")
                pickle.dump(self._indirect_jumps, open(dump_file_prefix + "_indirect_jumps.angr", "wb"))
                pickle.dump(self._cfg, open(dump_file_prefix + "_coerce_cfg.angr", "wb"))
                pickle.dump(self._unassured_functions, open(dump_file_prefix + "_unassured_functions.angr", "wb"))

        if len(self._indirect_jumps):
            # We got some indirect jumps!
            # Gotta execute each basic block and see where it wants to jump to
            function_starts = self._process_indirect_jumps()

            self.base_address = self._solve_for_base_address(function_starts, self._unassured_functions)

            l.info("Base address should be 0x%x", self.base_address)

        else:
            l.debug("No indirect jumps are found. We switch to the slowpath mode.")
            # TODO: Slowpath mode...
            while True:
                next_addr = self._get_next_code_addr(initial_state)
                percentage = self._seg_list.occupied_size * 100.0 / (self._valid_memory_region_size)
                l.info("Analyzing %xh, progress %0.04f%%", next_addr, percentage)
                if next_addr is None:
                    break

                self._call_map.add_node(next_addr)

                self._scan_code(traced_address, function_exits, initial_state, next_addr)

        # Post-processing: Map those calls that are not made by call/blr
        # instructions to their targets in our map
        for src, s in function_exits.items():
            if src in self._call_map:
                for target in s:
                    if target in self._call_map:
                        self._call_map.add_edge(src, target)

        nodes = sorted(self._call_map.nodes())
        for i in range(len(nodes) - 1):
            if nodes[i] >= nodes[i + 1] - 4:
                for dst in self._call_map.successors(nodes[i + 1]):
                    self._call_map.add_edge(nodes[i], dst)
                for src in self._call_map.predecessors(nodes[i + 1]):
                    self._call_map.add_edge(src, nodes[i])
                self._call_map.remove_node(nodes[i + 1])

        l.debug("Construction finished.")

    def _full_code_scan(self):
        """
        Perform a full code scan on the target binary.
        """

        # We gotta time this function
        start_time = datetime.now()

        traced_address = set()
        self._functions = set()
        self._call_map = networkx.DiGraph()
        self._cfg = networkx.DiGraph()
        initial_state = self._project.factory.blank_state(mode="fastpath")
        initial_options = initial_state.options - {simuvex.o.TRACK_CONSTRAINTS} - simuvex.o.refs
        initial_options |= {simuvex.o.SUPER_FASTPATH}
        # initial_options.remove(simuvex.o.COW_STATES)
        initial_state.options = initial_options
        # Sadly, not all calls to functions are explicitly made by call
        # instruction - they could be a jmp or b, or something else. So we
        # should record all exits from a single function, and then add
        # necessary calling edges in our call map during the post-processing
        # phase.
        function_exits = defaultdict(set)

        widgets = [progressbar.Percentage(),
                   ' ',
                   progressbar.Bar(marker=progressbar.RotatingMarker()),
                   ' ',
                   progressbar.Timer(),
                   ' ',
                   progressbar.ETA()
        ]

        pb = progressbar.ProgressBar(widgets=widgets, maxval=10000 * 100).start()

        while True:
            next_addr = self._get_next_code_addr(initial_state)
            percentage = self._seg_list.occupied_size * 100.0 / (self._valid_memory_region_size)
            if percentage > 100.0: percentage = 100.0
            pb.update(percentage * 10000)

            if next_addr is not None:
                l.info("Analyzing %xh, progress %0.04f%%", next_addr, percentage)
            else:
                l.info('No more addr to analyze. Progress %0.04f%%', percentage)
                break

            self._call_map.add_node(next_addr)

            self._scan_code(traced_address, function_exits, initial_state, next_addr)

        pb.finish()
        end_time = datetime.now()
        l.info("A full code scan takes %d seconds.", (end_time - start_time).seconds)

    def _calc_entropy(self, data, size=None):
        if not data:
            return 0
        entropy = 0
        if size is None: size = len(data)
        data = str(pyvex.ffi.buffer(data, size))
        for x in xrange(0, 256):
            p_x = float(data.count(chr(x)))/size
            if p_x > 0:
                entropy += - p_x * math.log(p_x, 2)
        return entropy

    def _dbg_output(self):
        ret = ""
        ret += "Functions:\n"
        function_list = list(self._functions)
        # Sort it
        function_list = sorted(function_list)
        for f in function_list:
            ret += "0x%08x" % f

        return ret

    def genenare_callmap_sif(self, filepath):
        """
        Generate a sif file from the call map
        """
        graph = self.call_map

        if graph is None:
            raise AngrGirlScoutError('Please generate the call graph first.')

        f = open(filepath, "wb")

        for src, dst in graph.edges():
            f.write("0x%x\tDirectEdge\t0x%x\n" % (src, dst))

        f.close()

    def generate_code_cover(self):
        """
        Generate a list of all recovered basic blocks.
        """

        lst = [ ]
        for irsb_addr in self._cfg.nodes():
            if irsb_addr not in self._block_size:
                continue
            irsb_size = self._block_size[irsb_addr]
            lst.append((irsb_addr, irsb_size))

        lst = sorted(lst, key=lambda x: x[0])

        return lst

from ..blade import Blade
from ..errors import AngrGirlScoutError, AngrTranslationError, AngrMemoryError



import sys
import pyvex
import simuvex
from archinfo import ArchARM

import logging
l = logging.getLogger("angr.lifter")

VEX_IRSB_MAX_SIZE = 400
VEX_IRSB_MAX_INST = 99
VEX_DEFAULT_OPT_LEVEL = 1

class Lifter:
    def __init__(self, project):
        self._project = project
        self._thumbable = isinstance(project.arch, ArchARM)

    def lift(self, addr, insn_bytes=None, max_size=None, num_inst=None,
             traceflags=0, thumb=False, backup_state=None, opt_level=None):
        """
        Returns a pyvex block starting at address addr

        @param addr: the address at which to start the block

        The below parameters are optional:
        @param thumb: whether the block should be lifted in ARM's THUMB mode
        @param backup_state: a state to read bytes from instead of using project memory
        @param opt_level: the VEX optimization level to use
        @param insn_bytes: a string of bytes to use for the block instead of the project
        @param max_size: the maximum size of the block, in bytes
        @param num_inst: the maximum number of instructions
        @param traceflags: traceflags to be passed to VEX. Default: 0
        """
        passed_max_size = max_size is not None
        passed_num_inst = num_inst is not None
        max_size = VEX_IRSB_MAX_SIZE if max_size is None else max_size
        num_inst = VEX_IRSB_MAX_INST if num_inst is None else num_inst
        opt_level = VEX_DEFAULT_OPT_LEVEL if opt_level is None else opt_level

        if self._thumbable and addr % 2 == 1:
            thumb = True
        elif not self._thumbable and thumb:
            l.warning("Why did you pass in thumb=True on a non-ARM architecture")
            thumb = False

        if thumb:
            addr &= ~1

        # TODO: FIXME: figure out what to do if we're about to exhaust the memory
        # (we can probably figure out how many instructions we have left by talking to IDA)

        if insn_bytes is not None:
            buff, size = insn_bytes, len(insn_bytes)
            max_size = min(max_size, size)
            passed_max_size = True
        else:
            buff, size = "", 0

            if backup_state:
                buff, size = self._bytes_from_state(backup_state, addr, max_size)
                max_size = min(max_size, size)
            else:
                try:
                    buff, size = self._project.loader.memory.read_bytes_c(addr)
                except KeyError:
                    pass

            if not buff or size == 0:
                raise AngrMemoryError("No bytes in memory for block starting at 0x%x." % addr)

        # deal with thumb mode in ARM, sending an odd address and an offset
        # into the string
        byte_offset = 0
        real_addr = addr
        if thumb:
            byte_offset = 1
            addr += 1

        l.debug("Creating pyvex.IRSB of arch %s at 0x%x", self._project.arch.name, addr)

        pyvex.set_iropt_level(opt_level)
        try:
            if passed_max_size and not passed_num_inst:
                irsb = pyvex.IRSB(bytes=buff,
                                  mem_addr=addr,
                                  num_bytes=max_size,
                                  arch=self._project.arch,
                                  bytes_offset=byte_offset,
                                  traceflags=traceflags)
            elif not passed_max_size and passed_num_inst:
                irsb = pyvex.IRSB(bytes=buff,
                                  mem_addr=addr,
                                  num_bytes=VEX_IRSB_MAX_SIZE,
                                  num_inst=num_inst,
                                  arch=self._project.arch,
                                  bytes_offset=byte_offset,
                                  traceflags=traceflags)
            elif passed_max_size and passed_num_inst:
                irsb = pyvex.IRSB(bytes=buff,
                                  mem_addr=addr,
                                  num_bytes=min(size, max_size),
                                  num_inst=num_inst,
                                  arch=self._project.arch,
                                  bytes_offset=byte_offset,
                                  traceflags=traceflags)
            else:
                irsb = pyvex.IRSB(bytes=buff,
                                  mem_addr=addr,
                                  num_bytes=min(size, max_size),
                                  arch=self._project.arch,
                                  bytes_offset=byte_offset,
                                  traceflags=traceflags)
        except pyvex.PyVEXError:
            l.debug("VEX translation error at %#x", addr)
            if isinstance(buff, str):
                l.debug('Using bytes: ' + buff)
            else:
                l.debug("Using bytes: " + str(pyvex.ffi.buffer(buff, size)).encode('hex'))
            e_type, value, traceback = sys.exc_info()
            raise AngrTranslationError, ("Translation error", e_type, value), traceback

        if insn_bytes is None:
            for stmt in irsb.statements:
                if stmt.tag != 'Ist_IMark' or stmt.addr == real_addr:
                    continue
                if self._project.is_hooked(stmt.addr):
                    size = stmt.addr - real_addr
                    irsb = pyvex.IRSB(bytes=buff, mem_addr=addr, num_bytes=size, arch=self._project.arch, bytes_offset=byte_offset, traceflags=traceflags)
                    break

        irsb = self._post_process(irsb)
        return Block(buff, irsb, thumb)

    @staticmethod
    def _bytes_from_state(backup_state, addr, max_size):
        arr = [ ]

        for i in range(addr, addr + max_size):
            if i in backup_state.memory:
                val = backup_state.memory.load(backup_state.BVV(i), backup_state.BVV(1))
                try:
                    val = backup_state.se.exactly_n_int(val, 1)[0]
                    val = chr(val)
                except simuvex.SimValueError:
                    break

                arr.append(val)
            else:
                break

        buff = "".join(arr)
        size = len(buff)

        return buff, size

    def _post_process(self, block):
        '''
        Do some post-processing work here.
        :param block:
        :return:
        '''

        block.statements = [ x for x in block.statements if x.tag != 'Ist_NoOp' ]

        funcname = "_post_process_%s" % self._project.arch.name
        if hasattr(self, funcname):
            block = getattr(self, funcname)(block)

        return block

    def _post_process_ARM(self, block):

        # Jumpkind
        if block.jumpkind == "Ijk_Boring":
            # If PC is moved to LR, then this should be an Ijk_Call
            #
            # Example:
            # MOV LR, PC
            # MOV PC, R8

            stmts = block.statements

            lr_store_id = None
            inst_ctr = 1
            for i, stmt in reversed(list(enumerate(stmts))):
                if isinstance(stmt, pyvex.IRStmt.Put):
                    if stmt.offset == self._project.arch.registers['lr'][0]:
                        lr_store_id = i
                        break
                if isinstance(stmt, pyvex.IRStmt.IMark):
                    inst_ctr += 1

            if lr_store_id is not None and inst_ctr == 2:
                block.jumpkind = "Ijk_Call"

        return block
    _post_process_ARMEL = _post_process_ARM
    _post_process_ARMHF = _post_process_ARM

    @staticmethod
    def _post_process_MIPS32(block):

        # Handle unconditional branches
        # `beq $zero, $zero, xxxx`
        # It is translated to
        #
        # 15 | ------ IMark(0x401684, 4, 0) ------
        # 16 | t0 = CmpEQ32(0x00000000, 0x00000000)
        # 17 | PUT(128) = 0x00401688
        # 18 | ------ IMark(0x401688, 4, 0) ------
        # 19 | if (t0) goto {Ijk_Boring} 0x401684
        # 20 | PUT(128) = 0x0040168c
        # 21 | t4 = GET:I32(128)
        # NEXT: PUT(128) = t4; Ijk_Boring
        #

        stmts = block.statements
        tmp_exit = None
        exit_stmt_idx = None
        dst = None

        for i, stmt in reversed(list(enumerate(stmts))):
            if tmp_exit is None:
                # Looking for the Exit statement
                if isinstance(stmt, pyvex.IRStmt.Exit) and \
                        isinstance(stmt.guard, pyvex.IRExpr.RdTmp):
                    tmp_exit = stmt.guard.tmp
                    dst = stmt.dst
                    exit_stmt_idx = i
            else:
                # Looking for the WrTmp statement
                if isinstance(stmt, pyvex.IRStmt.WrTmp) and \
                    stmt.tmp == tmp_exit:
                    if isinstance(stmt.data, pyvex.IRExpr.Binop) and \
                            stmt.data.op == 'Iop_CmpEQ32' and \
                            isinstance(stmt.data.child_expressions[0], pyvex.IRExpr.Const) and \
                            isinstance(stmt.data.child_expressions[1], pyvex.IRExpr.Const) and \
                            stmt.data.child_expressions[0].con.value == stmt.data.child_expressions[1].con.value:

                        # Create a new IRConst
                        irconst = pyvex.IRExpr.Const.__new__()      # XXX: does this work???
                        irconst.con = dst
                        irconst.is_atomic = True
                        irconst.result_type = dst.type
                        irconst.tag = 'Iex_Const'

                        block.statements = block.statements[ : exit_stmt_idx] + block.statements[exit_stmt_idx + 1 : ]
                        # Replace the default exit!
                        block.next = irconst

                    else:
                        break

        return block

    @staticmethod
    def _find_source(statements, put_stmt_id):
        '''
        Execute the statements backwards and figure out where the value comes from
        This is not a slicer. It only take care of a small portion of statement types.
        :param statements:
        :param put_stmt_id:
        :return:
        '''
        temps = set()
        src_stmt_ids = set()

        if not isinstance(statements[put_stmt_id],pyvex.IRStmt.Put):
            return None

        if not isinstance(statements[put_stmt_id].data, pyvex.IRExpr.RdTmp):
            return None

        temps.add(statements[put_stmt_id].data.tmp)

        for i in xrange(put_stmt_id, -1, -1):
            stmt = statements[i]
            if isinstance(stmt, pyvex.IRStmt.WrTmp):
                data = None
                if stmt.tmp in temps:
                    data = stmt.data
                if isinstance(data, pyvex.IRExpr.RdTmp):
                    temps.add(data.tmp)
                elif isinstance(data, pyvex.IRExpr.Get):
                    src_stmt_ids.add(i)
                    temps.remove(stmt.tmp)

        return src_stmt_ids

class Block(object):
    def __init__(self, byte_string, vex, thumb):
        self._bytes = byte_string
        self.vex = vex
        self._thumb = thumb
        self._arch = vex.arch
        self._capstone = None
        self.addr = None
        self.size = vex.size
        self.instructions = vex.instructions
        self.instruction_addrs = []

        for stmt in vex.statements:
            if stmt.tag != 'Ist_IMark':
                continue
            if self.addr is None:
                self.addr = stmt.addr
            self.instruction_addrs.append(stmt.addr)

        if self.addr is None:
            l.warning('Lifted basic block with no IMarks!')
            self.addr = 0

    def __repr__(self):
        return '<Block for %#x, %d bytes>' % (self.addr, self.size)

    def __getstate__(self):
        self._bytes = self.bytes
        return self.__dict__

    def __setstate__(self, data):
        self.__dict__.update(data)

    def pp(self):
        return self.capstone.pp()

    @property
    def bytes(self):
        bytestring = self._bytes
        if not isinstance(bytestring, str):
            bytestring = str(pyvex.ffi.buffer(bytestring, self.size))
        return bytestring

    @property
    def capstone(self):
        if self._capstone: return self._capstone

        cs = self._arch.capstone if not self._thumb else self._arch.capstone_thumb

        insns = []

        for cs_insn in cs.disasm(self.bytes, self.addr):
            insns.append(CapstoneInsn(cs_insn))
        block = CapstoneBlock(self.addr, insns, self._thumb, self._arch)

        self._capstone = block
        return block

class CopyClass:
    def __init__(self, obj):
        for attr in dir(obj):
            if attr.startswith('_'):
                continue
            val = getattr(obj, attr)
            if type(val) in (int, long, list, tuple, str, dict, float): # pylint: disable=unidiomatic-typecheck
                setattr(self, attr, val)
            else:
                setattr(self, attr, CopyClass(val))

class CapstoneInsn(object):
    def __init__(self, insn):
        self.address = insn.address
        self.bytes = insn.bytes
        if hasattr(insn, 'cc'):
            self.cc = insn.cc
        self.groups = insn.groups
        self.id = insn.id
        self._insn_name = insn.insn_name()
        self.mnemonic = insn.mnemonic
        self.op_str = insn.op_str
        self.operands = map(CopyClass, insn.operands)
        self.size = insn.size

    def group(self, grpnum):
        return grpnum in self.groups

    def insn_name(self):
        return self._insn_name

    def __str__(self):
        return "0x%x:\t%s\t%s" % (self.address, self.mnemonic, self.op_str)

    def __repr__(self):
        return '<CapstoneInsn "%s" for %#x>' % (self.mnemonic, self.address)

class CapstoneBlock(object):
    def __init__(self, addr, insns, thumb, arch):
        self.addr = addr
        self.insns = insns
        self.thumb = thumb
        self.arch = arch

    def pp(self):
        print str(self)

    def __str__(self):
        return '\n'.join(map(str, self.insns))

    def __repr__(self):
        return '<CapstoneBlock for %#x>' % self.addr


from .errors import AngrMemoryError, AngrTranslationError




class CodeLocation(object):
    """
    Stands for a specific program point by specifying basic block address and statement ID (for IRSBs), or SimProcedure
    name (for SimProcedures).
    """

    def __init__(self, simrun_addr, stmt_idx, sim_procedure=None):
        """
        Constructor.

        :param simrun_addr: Address of the SimRun
        :param stmt_idx: Statement ID. None for SimProcedures
        :param sim_procedure: The corresponding SimProcedure class.
        """

        self.simrun_addr = simrun_addr
        self.stmt_idx = stmt_idx
        self.sim_procedure = sim_procedure

    def __repr__(self):
        if self.simrun_addr is None:
            return '[%s]' % self.sim_procedure

        else:
            if self.stmt_idx is None:
                return "[%#x(-)]" % (self.simrun_addr)
            else:
                return "[%#x(%d)]" % (self.simrun_addr, self.stmt_idx)

    def __eq__(self, other):
        """
        Check if self is the same as other.
        """
        return self.simrun_addr == other.simrun_addr and self.stmt_idx == other.stmt_idx and \
               self.sim_procedure is other.sim_procedure

    def __hash__(self):
        """
        returns the hash value of self.
        """
        return hash((self.simrun_addr, self.stmt_idx, self.sim_procedure))



import copy
import logging
l = logging.getLogger("angr.path")

from os import urandom
import collections

import mulpyplexer

#pylint:disable=unidiomatic-typecheck

class CallFrame(object):
    """
        Instance variables:
        Int             from address
        Int             to address
        claripy.E       pointer to from_addr
    """
    def __init__(self, state):
        """
        Int-> CallFrame
        Create a new CallFrame with the given arguments
        """
        self.faddr = state.scratch.bbl_addr
        self.taddr = state.se.any_int(state.ip)
        self.sptr = state.regs.sp

        self.saved_return_address = self.get_return_address(state)
        self.saved_sp = state.regs.sp
        self.saved_bp = state.regs.bp

    @staticmethod
    def get_return_address(state):
        if state.arch.call_pushes_ret:
            return state.memory.load(state.regs.sp, state.arch.bits/8, endness=state.arch.memory_endness)
        else:
            return state.regs.lr

    def __repr__(self):
        """
        Returns a string representation of this callframe
        """
        return "0x%x (0x%x)" % (self.taddr, self.faddr)

class CallStack(object):
    """
        Instance variables:
        List        List of CallFrame
    """
    def __init__(self):
        """
        -> CallStack
        Create a new CallStack
        """
        self.callstack = []

    def __iter__(self):
        """
        -> Generator
        Overriding, for using the CallStack with iterators
        """
        for cf in self.callstack:
            yield cf

    def push(self, cf):
        """
        CallFrame ->
        Add the given CallFrame to the list of CallFrames
        """
        self.callstack.append(cf)

    def pop(self):
        """
        -> CallFrame
        Pop a CallFrame from the list
        """
        try:
            return self.callstack.pop()
        except IndexError:
            raise IndexError("pop from empty CallStack")

    def __getitem__(self, k):
        '''
        Returns the CallFrame at index k.
        '''
        return self.callstack[k]

    def __len__(self):
        """
        -> Int
        Return the length of the stuff
        """
        return len(self.callstack)

    def __repr__(self):
        """
        Better representation of this callstack
        """
        return "CallStack (depth %d) [ %s ]" % (len(self.callstack),
                                                ", ".join([ str(f) for f in self.callstack ]))

    def __eq__(self, other):
        """
        Compare two callstacks.
        """
        if len(self.callstack) != len(other.callstack):
            return False

        for c1, c2 in zip(self.callstack, other.callstack):
            if c1.faddr != c2.faddr or c1.taddr != c2.taddr:
                return False

        return True

    def __hash__(self):
        return hash(''.join([ '%d_%d' % (c.faddr, c.taddr) for c in self.callstack ]))

    def copy(self):
        """
        -> CallStack
        Make a copy of self
        """
        c = CallStack()
        c.callstack = list(self.callstack)
        return c

class Path(object):
    def __init__(self, project, state, path=None):
        # this is the state of the path
        self.state = state
        self.errored = False

        # project
        self._project = project

        # this path's information
        self.length = 0
        self.extra_length = 0

        # the previous run
        self.previous_run = None
        self.last_events = [ ]
        self.last_actions = [ ]
        self.fresh_variables = [ ]

        # the path history
        self.backtrace = [ ]
        self.addr_backtrace = [ ]
        self.targets = [ ]
        self.guards = [ ]
        self.sources = [ ]
        self.jumpkinds = [ ]
        self.events = [ ]
        self.actions = [ ]
        self.callstack = CallStack()
        self.popped_callframe = None
        self.blockcounter_stack = [ collections.Counter() ]

        # A custom information store that will be passed to all its descedents
        self.info = { }

        # for merging
        self._upcoming_merge_points = [ ]
        self._merge_flags = [ ]
        self._merge_values = [ ]
        self._merge_backtraces = [ ]
        self._merge_addr_backtraces = [ ]
        self._merge_depths = [ ]

        # copy a path if it's given
        if path is not None:
            self._record_path(path)

        # for printing/ID stuff and inheritence
        self.name = str(id(self))
        self.path_id = urandom(8).encode('hex')

        # Whitelist and last_stmt used for executing program slices
        self.stmt_whitelist = None
        self.last_stmt = None

        # actual analysis stuff
        self._run_args = None       # sim_run args, to determine caching
        self._run = None
        self._run_error = None
        self._reachable = None

        if self.state is not None:
            self._record_state(self.state)

    @property
    def addr(self):
        return self.state.se.any_int(self.state.regs.ip)

    #
    # Stepping methods and successor access
    #

    def step(self, **run_args):
        '''
        Step a path forward. Optionally takes any argument applicable
        to project.factory.sim_run:

        @param jumpkind: the jumpkind of the previous exit
        @param addr an address: to execute at instead of the state's ip
        @param stmt_whitelist: a list of stmt indexes to which to confine execution
        @param last_stmt: a statement index at which to stop execution
        @param thumb: whether the block should be lifted in ARM's THUMB mode
        @param backup_state: a state to read bytes from instead of using project memory
        @param opt_level: the VEX optimization level to use
        @param insn_bytes: a string of bytes to use for the block instead of the project
        @param max_size: the maximum size of the block, in bytes
        @param num_inst: the maximum number of instructions
        @param traceflags: traceflags to be passed to VEX. Default: 0
        '''
        if self._run_args != run_args or not self._run:
            self._run_args = run_args
            self._make_sim_run()

        if self._run_error:
            return [ ErroredPath(self._run_error, self._project, self.state.copy(), path=self) ]

        out = [ Path(self._project, s, path=self) for s in self._run.flat_successors ]
        if 'insn_bytes' in run_args and not 'addr' in run_args and len(out) == 1 \
                and isinstance(self._run, simuvex.SimIRSB) \
                and self.addr + self._run.irsb.size == out[0].state.se.any_int(out[0].state.regs.ip):
            out[0].state.regs.ip = self.addr
        return out

    def _make_sim_run(self):
        self._run = None
        self._run_error = None
        try:
            self._run = self._project.factory.sim_run(self.state, **self._run_args)
        except (AngrError, simuvex.SimError, claripy.ClaripyError) as e:
            l.debug("Catching exception", exc_info=True)
            self._run_error = e
        except (TypeError, ValueError, ArithmeticError, MemoryError) as e:
            l.debug("Catching exception", exc_info=True)
            self._run_error = e

    @property
    def next_run(self):
        if self._run_error:
            return None
        if not self._run:
            raise AngrPathError("Please call path.step() before accessing next_run")
        return self._run

    @property
    def successors(self):
        if not (self._run_error or self._run):
            raise AngrPathError("Please call path.step() before accessing successors")
        return self.step(**self._run_args)

    @property
    def unconstrained_successors(self):
        if self._run_error:
            return []
        if not self._run:
            raise AngrPathError("Please call path.step() before accessing successors")
        return [ Path(self._project, s, path=self) for s in self._run.unconstrained_successors ]

    @property
    def unsat_successors(self):
        if self._run_error:
            return []
        if not self._run:
            raise AngrPathError("Please call path.step() before accessing successors")
        return [ Path(self._project, s, path=self) for s in self._run.unsat_successors ]

    @property
    def mp_successors(self):
        return mulpyplexer.MP(self.successors)

    @property
    def nonflat_successors(self):
        if self._run_error:
            return []
        if not self._run:
            raise AngrPathError("Please call path.step() before accessing successors")

        nonflat_successors = [ ]
        for s in self._run.successors + self._run.unconstrained_successors:
            sp = Path(self._project, s, path=self)
            nonflat_successors.append(sp)
        return nonflat_successors

    @property
    def unconstrained_successor_states(self):
        if self._run_error:
            return []
        if not self._run:
            raise AngrPathError("Please call path.step() before accessing successors")

        return self._run.unconstrained_successors

    #
    # Utility functions
    #

    def trim_history(self):
        '''
        Trims a path's history (removes actions, etc).
        '''

        #self.backtrace = self.backtrace[-1:]
        #self.addr_backtrace = self.addr_backtrace[-1:]
        #self.jumpkinds = self.jumpkinds[-1:]
        self.targets = self.targets[-1:]
        self.guards = self.guards[-1:]
        self.sources = self.sources[-1:]
        self.events = self.events[-1:]
        self.actions = self.actions[-1:]

    def divergence_addr(self, other):
        '''
        Returns the basic block at which the paths diverged.

        @param other: the other Path
        @returns an address (long)
        '''

        for i in range(max([len(self.addr_backtrace), len(other.addr_backtrace)])):
            if i > len(self.addr_backtrace):
                return other.addr_backtrace[i-1]
            elif i > len(other.addr_backtrace):
                return self.addr_backtrace[i-1]
            elif self.addr_backtrace[i] != other.addr_backtrace[i]:
                return self.addr_backtrace[i-1]


    def detect_loops(self, n=None): #pylint:disable=unused-argument
        '''
        Returns the current loop iteration that a path is on.

        @param n: the minimum number of iterations to check for.
        @returns the number of the loop iteration it's in
        '''

        # TODO: make this work better
        #addr_strs = [ "%x"%x for x in self.addr_backtrace ]
        #bigstr = "".join(addr_strs)

        #candidates = [ ]

        #max_iteration_length = len(self.addr_backtrace) / n
        #for i in range(max_iteration_length):
        #   candidates.append("".join(addr_strs[-i-0:]))

        #for c in reversed(candidates):
        #   if bigstr.count(c) >= n:
        #       return n
        #return None

        mc = self.blockcounter_stack[-1].most_common()
        if len(mc) == 0:
            return None
        else:
            return mc[0][1]

    #
    # Error checking
    #

    _jk_errors = set(("Ijk_EmFail", "Ijk_NoDecode", "Ijk_MapFail"))
    _jk_signals = set(('Ijk_SigILL', 'Ijk_SigTRAP', 'Ijk_SigSEGV', 'Ijk_SigBUS',
                       'Ijk_SigFPE_IntDiv', 'Ijk_SigFPE_IntOvf'))
    _jk_all_bad = _jk_errors | _jk_signals

    #
    # Convenience functions
    #

    @property
    def reachable(self):
        return self._reachable if self._reachable is not None else self.state.satisfiable()

    @property
    def weighted_length(self):
        return self.length + self.extra_length

    @property
    def jumpkind(self):
        return self.state.scratch.jumpkind

    @property
    def _s0(self):
        return self.step()[0]
    @property
    def _s1(self):
        return self.step()[1]
    @property
    def _s2(self):
        return self.step()[2]
    @property
    def _s3(self):
        return self.step()[3]

    #
    # State continuation
    #

    def _record_path(self, path):
        self.last_events = list(path.last_events)
        self.last_actions = list(path.last_actions)
        self.events.extend(path.events)
        self.actions.extend(path.actions)

        self.backtrace.extend(path.backtrace)
        self.addr_backtrace.extend(path.addr_backtrace)
        self.callstack.callstack.extend(path.callstack.callstack)
        self.popped_callframe = path.popped_callframe

        self.guards.extend(path.guards)
        self.sources.extend(path.sources)
        self.jumpkinds.extend(path.jumpkinds)
        self.targets.extend(path.targets)

        self.length = path.length
        self.extra_length = path.extra_length
        self.previous_run = path._run

        self.info = { k:copy.copy(v) for k, v in path.info.iteritems() }

        self.blockcounter_stack = [ collections.Counter(s) for s in path.blockcounter_stack ]
        self._upcoming_merge_points = list(path._upcoming_merge_points)
        self._merge_flags = list(path._merge_flags)
        self._merge_values = list(path._merge_values)
        self._merge_backtraces = list(path._merge_backtraces)
        self._merge_addr_backtraces = list(path._merge_addr_backtraces)
        self._merge_depths = list(path._merge_depths)

        # if a run is provided, record it
        if path._run is not None:
            self._record_run(path._run)


    def _record_state(self, state):
        '''
        Adds the information from the last run to the current path.
        '''

        l.debug("Extending path with state %s", state)

        self.last_events = list(state.log.events)
        self.last_actions = list(e for e in state.log.events if isinstance(e, simuvex.SimAction))

        if simuvex.o.TRACK_ACTION_HISTORY in state.options:
            self.events.extend(self.last_events)
            self.actions.extend(self.last_actions)

        self.jumpkinds.append(state.scratch.jumpkind)
        self.targets.append(state.scratch.target)
        self.guards.append(state.scratch.guard)
        self.sources.append(state.scratch.source)

        # maintain the blockcounter stack
        if self.jumpkinds[-1] == "Ijk_Call":
            l.debug("... it's a call!")
            callframe = CallFrame(state)
            self.callstack.push(callframe)
            self.blockcounter_stack.append(collections.Counter())
        elif self.jumpkinds[-1].startswith('Ijk_Sys'):
            l.debug("... it's a syscall!")
            callframe = CallFrame(state)
            self.callstack.push(callframe)
            self.blockcounter_stack.append(collections.Counter())
        elif self.jumpkinds[-1] == "Ijk_Ret":
            l.debug("... it's a ret!")
            self.blockcounter_stack.pop()
            if len(self.blockcounter_stack) == 0:
                l.debug("... WARNING: unbalanced callstack")
                self.blockcounter_stack.append(collections.Counter())

            if len(self.callstack) > 0:
                self.popped_callframe = self.callstack.pop()

        self.addr_backtrace.append(state.scratch.bbl_addr)
        self.blockcounter_stack[-1][state.scratch.bbl_addr] += 1
        self.length += 1

    def _record_run(self, run):
        '''
        Adds the information from the last run to the current path.
        '''
        l.debug("Extending path with run %s", run)

        # maintain the blockstack
        self.backtrace.append(str(run))

    #
    # Merging and splitting
    #

    def unmerge(self):
        '''
        Unmerges the state back into different possible states.
        '''

        l.debug("Unmerging %s!", self)

        states = [ self.state ]

        for flag,values in zip(self._merge_flags, self._merge_values):
            l.debug("... processing %s with %d possibilities", flag, len(values))

            new_states = [ ]

            for v in values:
                for s in states:
                    s_copy = s.copy()
                    s_copy.add_constraints(flag == v)
                    new_states.append(s_copy)

            states = [ s for s in new_states if s.satisfiable() ]
            l.debug("... resulting in %d satisfiable states", len(states))

        new_paths = [ ]
        for s in states:
            s.simplify()

            p = Path(self._project, s, path=self)
            new_paths.append(p)
        return new_paths

    def merge(self, *others):
        '''
        Returns a merger of this path with *others.
        '''
        all_paths = list(others) + [ self ]
        if len(set([ o.addr for o in all_paths])) != 1:
            raise AngrPathError("Unable to merge paths.")

        # merge the state
        new_state, merge_flag, _ = self.state.merge(*[ o.state for o in others ])
        new_path = Path(self._project, new_state, path=self)

        # fix the backtraces
        divergence_index = [ len(set(addrs)) == 1 for addrs in zip(*[ o.addr_backtrace for o in all_paths ]) ].index(False)
        new_path.addr_backtrace = self.addr_backtrace[:divergence_index]
        new_path.addr_backtrace.append(-1)
        new_path.backtrace = self.backtrace[:divergence_index]
        new_path.backtrace.append(("MERGE POINT: %s" % merge_flag))

        # reset the upcoming merge points
        new_path._upcoming_merge_points = [ ]
        new_path._merge_flags.append(merge_flag)
        new_path._merge_values.append(list(range(len(all_paths))))
        new_path._merge_backtraces.append( [ o.backtrace for o in all_paths ] )
        new_path._merge_addr_backtraces.append( [ o.addr_backtrace for o in all_paths ] )
        new_path._merge_depths.append(new_path.length) #

        return new_path

    def copy(self):
        p = Path(self._project, self.state.copy())

        p.last_events = list(self.last_events)
        p.last_actions = list(self.last_actions)
        p.events = list(self.events)
        p.actions = list(self.actions)

        p.backtrace = list(self.backtrace)
        p.addr_backtrace = list(self.addr_backtrace)
        p.callstack = self.callstack.copy()
        p.popped_callframe = self.popped_callframe

        p.guards = list(self.guards)
        p.sources = list(self.sources)
        p.jumpkinds = list(self.jumpkinds)
        p.targets = list(self.targets)

        p.length = self.length
        p.extra_length = self.extra_length
        p.previous_run = self.previous_run
        p._run = self._run

        p.info = {k: copy.copy(v) for k, v in self.info.iteritems()}

        p.blockcounter_stack = [collections.Counter(s) for s in self.blockcounter_stack]
        p._upcoming_merge_points = list(self._upcoming_merge_points)
        p._merge_flags = list(self._merge_flags)
        p._merge_values = list(self._merge_values)
        p._merge_backtraces = list(self._merge_backtraces)
        p._merge_addr_backtraces = list(self._merge_addr_backtraces)
        p._merge_depths = list(self._merge_depths)

        return p

    def filter_actions(self, block_addr=None, block_stmt=None, insn_addr=None, read_from=None, write_to=None):
        if read_from is not None:
            if write_to is not None:
                raise ValueError("Can't handle read_from and write_to at the same time!")
            if read_from in ('reg', 'mem'):
                read_type = read_from
                read_offset = None
            elif isinstance(read_from, str):
                read_type = 'reg'
                read_offset = self._project.arch.registers[read_from][0]
            else:
                read_type = 'mem'
                read_offset = read_from
        if write_to is not None:
            if write_to in ('reg', 'mem'):
                write_type = write_to
                write_offset = None
            elif isinstance(write_to, str):
                write_type = 'reg'
                write_offset = self._project.arch.registers[write_to][0]
            else:
                write_type = 'mem'
                write_offset = write_to

        def addr_of_stmt(bbl_addr, stmt_idx):
            if stmt_idx is None:
                return None
            stmts = self._project.factory.block(bbl_addr).vex.statements
            if stmt_idx >= len(stmts):
                return None
            for i in reversed(xrange(stmt_idx + 1)):
                if stmts[i].tag == 'Ist_IMark':
                    return stmts[i].addr + stmts[i].delta
            return None

        def action_reads(action):
            if action.type != read_type:
                return False
            if action.action != 'read':
                return False
            if read_offset is None:
                return True
            addr = action.addr
            if isinstance(addr, simuvex.SimActionObject):
                addr = addr.ast
            if isinstance(addr, claripy.Base):
                if addr.symbolic:
                    return False
                addr = addr.model.value
            if addr != read_offset:
                return False
            return True

        def action_writes(action):
            if action.type != write_type:
                return False
            if action.action != 'write':
                return False
            if write_offset is None:
                return True
            addr = action.addr
            if isinstance(addr, simuvex.SimActionObject):
                addr = addr.ast
            if isinstance(addr, claripy.Base):
                if addr.symbolic:
                    return False
                addr = addr.model.value
            if addr != write_offset:
                return False
            return True

        return [x for x in self.actions if
                    (block_addr is None or x.bbl_addr == block_addr) and
                    (block_stmt is None or x.stmt_idx == block_stmt) and
                    (read_from is None or action_reads(x)) and
                    (write_to is None or action_writes(x)) and
                    (insn_addr is None or addr_of_stmt(x.bbl_addr, x.stmt_idx) == insn_addr)
            ]

    def __repr__(self):
        return "<Path with %d runs (at 0x%x)>" % (len(self.backtrace), self.addr)

class ErroredPath(Path):
    def __init__(self, error, *args, **kwargs):
        super(ErroredPath, self).__init__(*args, **kwargs)
        self.error = error
        self.errored = True

    def __repr__(self):
        return "<Errored Path with %d runs (at 0x%x, %s)>" % \
            (len(self.backtrace), self.addr, type(self.error).__name__)

    def step(self, *args, **kwargs):
        # pylint: disable=unused-argument
        raise AngrPathError("Cannot step forward an errored path")

    def retry(self, **kwargs):
        self._run_args = kwargs
        self._run = self._project.factory.sim_run(self.state, **self._run_args)
        return super(ErroredPath, self).step(**kwargs)

    def _record_state(self, *args, **kwargs):
        # pylint: disable=unused-argument
        pass

    def _record_run(self, *args, **kwargs):
        # pylint: disable=unused-argument
        pass


def make_path(project, runs):
    """
    A helper function to generate a correct angr.Path from a list of runs corresponding
    to a program path.

    We expect @runs to be a list of simruns corresponding to a program path
    """

    if len(runs) == 0:
        raise AngrPathError("Cannot generate Path from empty set of runs")

    # This creates a path which state is the the first run
    a_p = Path(project, runs[0].initial_state)
    # And records the first node's run
    a_p._record_run(runs[0])

    # We then go through all the nodes except the last one
    for r in runs[1:-1]:
        a_p._record_state(r.initial_state)
        a_p._record_run(r)

    # We record the last state and set it as current (it is the initial
    # state of the next run).
    a_p._record_state(runs[-1].initial_state)
    a_p.state = runs[-1].initial_state

    return a_p

from .errors import AngrError, AngrPathError
import simuvex
import claripy



import re
import logging
from collections import defaultdict

from archinfo import all_arches

from ..analysis import Analysis

l = logging.getLogger("angr.analyses.boyscout")

class BoyScout(Analysis):
    '''
    Try to determine the architecture and endieness of a binary blob
    '''
    def __init__(self, cookiesize=1):
        self.arch = None
        self.endianness = None
        self.votes = None
        self.cookiesize = cookiesize

        self._reconnoiter()

    def _reconnoiter(self):
        '''
        The implementation here is simple - just perform a pattern matching of all different architectures we support,
        and then perform a vote.
        :return: None
        '''

        # Retrieve the binary string of main binary
        strides = self._p.loader.main_bin.memory.stride_repr

        votes = defaultdict(int)

        for arch in all_arches:
            regexes = set()
            for ins_regex in arch.function_prologs.union(arch.function_epilogs):
                r = re.compile(ins_regex)
                regexes.add(r)

            for start_, _, data in strides:
                for regex in regexes:
                    # Match them!
                    for mo in regex.finditer(data):
                        position = mo.start() + start_
                        if position % arch.instruction_alignment == 0:
                            votes[(arch.name, arch.memory_endness)] += 1

            l.debug("%s %s hits %d times", arch.name, arch.memory_endness,
                    votes[(arch.name, arch.memory_endness)])

        arch_name, endianness, hits = sorted([(k[0], k[1], v) for k, v in votes.iteritems()], key=lambda x: x[2], reverse=True)[0]

        if hits < self.cookiesize * 2:
        # this cannot possibly be code
            arch_name = "DATA"
            endianness = ""

        self.arch = arch_name
        self.endianness = endianness
        # Save it as well for debugging
        self.votes = votes

        l.debug("The architecture should be %s with %s", self.arch, self.endianness)



from distutils.core import setup

setup(
    name='angr',
    version='4.5.9.13.post1',
    description='The next-generation binary analysis platform from UC Santa Barbara\'s Seclab!',
    packages=['angr', 'angr.surveyors', 'angr.analyses'],
    install_requires=[
        'capstone',
        'networkx',
        'futures',
        'progressbar',
        'mulpyplexer',
        'cooldict',
        'ana',
        'archinfo',
        'pyvex',
        'claripy',
        'simuvex',
        'cle',
    ],
)



import logging
from collections import defaultdict

import networkx

from simuvex import SimRegisterVariable, SimMemoryVariable

from ..errors import AngrDDGError
from ..analysis import Analysis
from .code_location import CodeLocation

l = logging.getLogger("angr.analyses.ddg")

class DDG(Analysis):
    """
    This is a fast data dependence graph directly gerenated from our CFG analysis result. The only reason for its
    existance is the speed. There is zero guarantee for being sound or accurate. You are supposed to use it only when
    you want to track the simplest data dependence, and you do not care about soundness or accuracy.

    For a better data dependence graph, please consider to perform a better static analysis first (like Value-set
    Analysis), and then construct a dependence graph on top of the analysis result (for example, the VFG in angr).

    Also note that since we are using states from CFG, any improvement in analysis performed on CFG (like a points-to
    analysis) will directly benefit the DDG.
    """
    def __init__(self, cfg, start=None, keep_data=False):
        """
        The constructor.

        :param cfg: Control flow graph. Please make sure each node has an associated `state` with it. You may want to
                generate your CFG with `keep_state`=True.
        :param start: an address, specifies where we start the generation of this data dependence graph.
        """
        self._project = self._p
        self._cfg = cfg
        self._start = self._project.entry if start is None else start

        self._graph = networkx.DiGraph()
        self._symbolic_mem_ops = set()

        self.keep_data = keep_data

        # Begin construction!
        self._construct()

    #
    # Properties
    #

    @property
    def graph(self):
        """
        :return: A networkx DiGraph instance representing the data dependence graph.
        """
        return self._graph

    def pp(self):
        """
        Pretty printing.
        """
        # TODO: make it prettier
        for k,v in self.graph.iteritems():
            for st, tup in v.iteritems():
                print "(0x%x, %d) <- (0x%x, %d)" % (k, st,
                                                            list(tup)[0][0],
                                                            list(tup)[0][1])

    def dbg_repr(self):
        """
        Representation for debugging.
        """
        # TODO:
        return str(self._graph)

    def __contains__(self, code_location):
        """
        If code_location is in the graph

        :param code_location: A CodeLocation instance
        :return: True/False
        """

        return code_location in self.graph

    #
    # Public methods
    #

    def get_predecessors(self, code_location):
        """
        Returns all predecessors of the code location

        :param code_location: A CodeLocation instance
        :return: a list of all predecessors
        """

        return self._graph.predecessors(code_location)

    #
    # Private methods
    #

    def _construct(self):
        """
        Construct the data dependence graph.

        We track the following types of dependence:
        - (Intra-IRSB) temporary variable dependencies
        - Register dependencies
        - Memory dependencies, although it's very limited. See below.

        We track the following types of memory access:
        - (Intra-functional) Stack read/write.
            Trace changes of stack pointers inside a function, and the dereferences of stack pointers.
        - (Inter-functional) Stack read/write.
        - (Global) Static memory positions.
            Keep a map of all accessible memory positions to their source statements per function. After that, we
            traverse the CFG and link each pair of reads/writes together in the order of control-flow.

        We do not track the following types of memory access
        - Symbolic memory access
            Well, they cannot be tracked under fastpath mode (which is the mode we are generating the CTF) anyways.
        """

        # TODO: Here we are assuming that there is only one node whose address is the entry point. Otherwise it should
        # TODO: be fixed.
        initial_node = self._cfg.get_any_node(self._start)

        # Initialize the worklist
        worklist = list(networkx.dfs_successors(self._cfg.graph, initial_node))
        # Also create a set for our worklist for fast inclusion test
        worklist_set = set(worklist)

        # A dict storing defs set
        # variable -> locations
        live_defs_per_node = {}

        while worklist:
            # Pop out a node
            node = worklist[0]
            worklist = worklist[ 1 : ]
            worklist_set.remove(node)

            # Grab all final states. There are usually more than one (one state for each successor), and we gotta
            # process all of them
            final_states = node.final_states

            if node in live_defs_per_node:
                live_defs = live_defs_per_node[node]
            else:
                live_defs = {}
                live_defs_per_node[node] = live_defs

            successing_nodes = self._cfg.graph.successors(node)
            for state in final_states:
                if state.scratch.jumpkind == 'Ijk_FakeRet' and len(final_states) > 1:
                    # Skip fakerets if there are other control flow transitions available
                    continue

                # TODO: Match the jumpkind
                # TODO: Support cases where IP is undecidable
                corresponding_successors = [n for n in successing_nodes if n.addr == state.se.any_int(state.ip)]
                if not corresponding_successors:
                    continue
                successing_node = corresponding_successors[0]

                new_defs = self._track(state, live_defs)

                if successing_node in live_defs_per_node:
                    defs_for_next_node = live_defs_per_node[successing_node]
                else:
                    defs_for_next_node = {}
                    live_defs_per_node[successing_node] = defs_for_next_node

                changed = False
                for var, code_loc_set in new_defs.iteritems():
                    if var not in defs_for_next_node:
                        l.debug('%s New var %s', state.ip, var)
                        defs_for_next_node[var] = code_loc_set
                        changed = True

                    else:
                        for code_loc in code_loc_set:
                            if code_loc not in defs_for_next_node[var]:
                                l.debug('%s New code location %s', state.ip, code_loc)
                                defs_for_next_node[var].add(code_loc)
                                changed = True

                if changed:
                    # Put all reachable successors back to our worklist again
                    if successing_node not in worklist_set:
                        worklist.append(successing_node)
                        worklist_set.add(successing_node)

                    all_successors_dict = networkx.dfs_successors(self._cfg._graph, source=successing_node)
                    for successors in all_successors_dict.values():
                        for s in successors:
                            if successing_node not in worklist_set:
                                worklist.append(s)
                                worklist_set.add(s)

    def _track(self, state, live_defs):
        """
        Given all live definitions prior to this program point, track the changes, and return a new list of live
        definitions. We scan through the action list of the new state to track the changes.

        :param state: The input state at that program point.
        :param live_defs: A list of all live definitions prior to reaching this program point.
        :return: A list of new live definitions.
        """

        # Make a copy of live_defs
        live_defs = live_defs.copy()

        action_list = list(state.log.actions)

        # Since all temporary variables are local, we simply track them in a local dict
        temps = {}

        # All dependence edges are added to the graph either at the end of this method, or when they are going to be
        # overwritten by a new edge. This is because we sometimes have to modify a  previous edge (e.g. add new labels
        # to the edge)
        temps_to_edges = defaultdict(list)
        regs_to_edges = defaultdict(list)

        def _annotate_edges_in_dict(dict_, key, **new_labels):
            """

            :param dict_: The dict, can be either `temps_to_edges` or `regs_to_edges`
            :param key: The key used in finding elements in the dict
            :param new_labels: New labels to be added to those edges
            """

            for edge_tuple in dict_[key]:
                # unpack it
                _, _, labels = edge_tuple
                for k, v in new_labels.iteritems():
                    if k in labels:
                        labels[k] = labels[k] + (v,)
                    else:
                        # Construct a tuple
                        labels[k] = (v,)

        def _dump_edge_from_dict(dict_, key, del_key=True):
            """
            Pick an edge from the dict based on the key specified, add it to our graph, and remove the key from dict.

            :param dict_: The dict, can be either `temps_to_edges` or `regs_to_edges`
            :param key: The key used in finding elements in the dict
            """
            for edge_tuple in dict_[key]:
                # unpack it
                prev_code_loc, current_code_loc, labels = edge_tuple
                # Add the new edge
                self._add_edge(prev_code_loc, current_code_loc, **labels)

            # Clear it
            if del_key:
                del dict_[key]

        for a in action_list:

            if a.bbl_addr is None:
                current_code_loc = CodeLocation(None, None, sim_procedure=a.sim_procedure)
            else:
                current_code_loc = CodeLocation(a.bbl_addr, a.stmt_idx)

            if a.type == "mem":
                if a.actual_addrs is None:
                    # For now, mem reads don't necessarily have actual_addrs set properly
                    addr_list = { state.se.any_int(a.addr.ast) }
                else:
                    addr_list = set(a.actual_addrs)

                for addr in addr_list:
                    variable = SimMemoryVariable(addr, a.data.ast.size())  # TODO: Properly unpack the SAO

                    if a.action == "read":
                        # Create an edge between def site and use site

                        prevdefs = self._def_lookup(live_defs, variable)

                        for prev_code_loc, labels in prevdefs.iteritems():
                            self._read_edge = True
                            self._add_edge(prev_code_loc, current_code_loc, **labels)

                    if a.action == "write":
                        # Kill the existing live def
                        self._kill(live_defs, variable, current_code_loc)

                    # For each of its register dependency and data dependency, we revise the corresponding edge
                    for reg_off in a.addr.reg_deps:
                        _annotate_edges_in_dict(regs_to_edges, reg_off, subtype='mem_addr')
                    for tmp in a.addr.tmp_deps:
                        _annotate_edges_in_dict(temps_to_edges, tmp, subtype='mem_addr')

                    for reg_off in a.data.reg_deps:
                        _annotate_edges_in_dict(regs_to_edges, reg_off, subtype='mem_data')
                    for tmp in a.data.tmp_deps:
                        _annotate_edges_in_dict(temps_to_edges, tmp, subtype='mem_data')

            elif a.type == 'reg':
                # For now, we assume a.offset is not symbolic
                # TODO: Support symbolic register offsets

                variable = SimRegisterVariable(a.offset, a.data.ast.size())

                if a.action == 'read':
                    # What do we want to do?
                    prevdefs = self._def_lookup(live_defs, variable)

                    if a.offset in regs_to_edges:
                        _dump_edge_from_dict(regs_to_edges, a.offset)

                    for prev_code_loc, labels in prevdefs.iteritems():
                        edge_tuple = (prev_code_loc, current_code_loc, labels)
                        regs_to_edges[a.offset].append(edge_tuple)

                else:
                    # write
                    self._kill(live_defs, variable, current_code_loc)

            elif a.type == 'tmp':
                # tmp is definitely not symbolic
                if a.action == 'read':
                    prev_code_loc = temps[a.tmp]
                    edge_tuple = (prev_code_loc, current_code_loc, {'type': 'tmp', 'data': a.tmp})

                    if a.tmp in temps_to_edges:
                        _dump_edge_from_dict(temps_to_edges, a.tmp)

                    temps_to_edges[a.tmp].append(edge_tuple)

                else:
                    # write
                    temps[a.tmp] = current_code_loc

            elif a.type == 'exit':
                # exits should only depend on tmps

                for tmp in a.tmp_deps:
                    prev_code_loc = temps[tmp]
                    edge_tuple = (prev_code_loc, current_code_loc, {'type': 'exit', 'data': tmp})

                    if tmp in temps_to_edges:
                        _dump_edge_from_dict(temps_to_edges, tmp)

                    temps_to_edges[tmp].append(edge_tuple)

        # In the end, dump all other edges in those two dicts
        for reg_offset in regs_to_edges:
            _dump_edge_from_dict(regs_to_edges, reg_offset, del_key=False)
        for tmp in temps_to_edges:
            _dump_edge_from_dict(temps_to_edges, tmp, del_key=False)

        return live_defs

    def _def_lookup(self, live_defs, variable):
        """
        This is a backward lookup in the previous defs.
        @addr_list is a list of normalized addresses.
        Note that, as we are using VSA, it is possible that @a is affected by
        several definitions.
        Returns: a dict {stmt:labels} where label is the number of individual
        addresses of @addr_list (or the actual set of addresses depending on the
        keep_addrs flag) that are definted by stmt.
        """

        prevdefs = {}

        if variable in live_defs:
            code_loc_set = live_defs[variable]
            for code_loc in code_loc_set:
                # Label edges with cardinality or actual sets of addresses
                if isinstance(variable, SimMemoryVariable):
                    type_ = 'mem'
                elif isinstance(variable, SimRegisterVariable):
                    type_ = 'reg'
                else:
                    raise AngrDDGError('Unknown variable type %s' % type(variable))

                if self.keep_data is True:
                    data = variable

                    prevdefs[code_loc] = {
                        'type': type_,
                        'data': data
                    }

                else:
                    if code_loc in prevdefs:
                        count = prevdefs[code_loc]['count'] + 1
                    else:
                        count = 0
                    prevdefs[code_loc] = {
                        'type': type_,
                        'count': count
                    }
        return prevdefs

    def _kill(self, live_defs, variable, code_loc):
        """
        Kill previous defs. @addr_list is a list of normalized addresses
        """

        # Case 1: address perfectly match, we kill
        # Case 2: a is a subset of the original address
        # Case 3: a is a superset of the original address

        live_defs[variable] = {code_loc}
        #l.debug("XX CodeLoc %s kills variable %s", code_loc, variable)

    def _add_edge(self, s_a, s_b, **edge_labels):
        """
         Add an edge in the graph from @s_a to statment @s_b, where @s_a and
         @s_b are tuples of statements of the form (irsb_addr, stmt_idx)
        """
        # Is that edge already in the graph ?
        # If at least one is new, then we are not redoing the same path again
        if (s_a, s_b) not in self.graph.edges():
            self.graph.add_edge(s_a, s_b, **edge_labels)
            self._new = True
            l.info("New edge: %s --> %s", s_a, s_b)



import itertools
import logging
from collections import defaultdict

from simuvex import SimIRSB, SimProcedure

l = logging.getLogger(name="angr.variableseekr")

class Variable(object):
    def __init__(self, idx, size, irsb_addr, stmt_id, ins_addr, custom_name=None, type_=None):
        self._idx = idx
        self._irsb_addr = irsb_addr
        self._ins_addr = ins_addr
        self._stmt_id = stmt_id
        self._size = size
        self._custom_name = custom_name
        self._type = type_

        # A model used to model the possible value range of this variable
        self._model = None

    @property
    def irsb_addr(self):
        return self._irsb_addr

    @property
    def stmt_id(self):
        return self._stmt_id

    @property
    def name(self):
        return "var_%d" % self._idx

    @property
    def type(self):
        return self._type

    @property
    def decl(self):
        if self._type is not None:
            return "{} {}".format(self._type, self)
        else:
            return str(self)

    @property
    def model(self):
        return self._model

    @model.setter
    def model(self, value):
        self._model = value

    @property
    def upper_bound(self):
        if self._model is not None:
            return self._model.upper_bound
        else:
            return None

    @property
    def lower_bound(self):
        if self._model is not None:
            return self._model.lower_bound
        else:
            return None

    @property
    def stride(self):
        if self._model is not None:
            return self._model.stride
        else:
            return None

    def __repr__(self):
        if self._custom_name is not None:
            return self._custom_name
        else:
            return self.name

class DummyVariable(Variable):
    def __init__(self, idx, size, irsb_addr, stmt_id, ins_addr):
        Variable.__init__(self, idx, size, irsb_addr, stmt_id, ins_addr)

    @property
    def name(self):
        return "dummy_var_%d" % self._idx

class RegisterVariable(Variable):
    def __init__(self, idx, size, irsb_addr, stmt_id, ins_addr, offset, custom_name=None, type_=None):
        Variable.__init__(self, idx, size, irsb_addr, stmt_id, ins_addr, custom_name, type_)
        self._offset = offset

    @property
    def name(self):
        return 'reg_var_%d' % self._idx

class StackVariable(Variable):
    '''
    _offset refers to the offset from stack base
    '''
    def __init__(self, idx, size, irsb_addr, stmt_id, ins_addr, offset, addr, type_=None):
        if type(size) not in [int, long]:
            import ipdb; ipdb.set_trace()
        Variable.__init__(self, idx, size, irsb_addr, stmt_id, ins_addr, type_)
        self._offset = offset
        self._addr = addr

    @property
    def idx(self):
        return self._idx

    @property
    def offset(self):
        return self._offset

    @offset.setter
    def offset(self, value):
        self._offset = value

    def detail_str(self):
        '''
        FIXME: This is a temporary fix for simuvex Issue #31
        '''
        if type(self._size) in [int, long]:
            s = 'StackVar %d [%s|%d] <irsb 0x%x, stmt %d> (%s-%s,%s)' % (self._idx, hex(self._offset), self._size, self._irsb_addr, self._stmt_id, self.lower_bound, self.upper_bound, self.stride)
        else:
            s = 'StackVar %d [%s|%s] <irsb 0x%x, stmt %d> (%s-%s,%s)' % (self._idx, hex(self._offset), self._size, self._irsb_addr, self._stmt_id, self.lower_bound, self.upper_bound, self.stride)
        return s

class VariableManager(object):
    def __init__(self, func_addr):
        self._func_addr = func_addr
        self._var_list = []
        self._stmt_to_var_map = {} # Maps a tuple of (irsb_addr, stmt_id) to the corresponding variable
        self._stack_variable_map = defaultdict(dict) # Maps stack offset to a stack variable

    def add(self, var):
        self._var_list.append(var)

        tpl = (var.irsb_addr, var.stmt_id)
        self._stmt_to_var_map[tpl] = var
        if isinstance(var, StackVariable):
            self._stack_variable_map[var.offset][tpl] = var

    def add_ref(self, var, irsb_addr, stmt_id):
        tpl = (irsb_addr, stmt_id)
        self._stmt_to_var_map[tpl] = var

    def get(self, irsb_addr, stmt_id):
        tpl = (irsb_addr, stmt_id)
        if tpl in self._stmt_to_var_map:
            return self._stmt_to_var_map[tpl]
        else:
            return None

    def get_stack_variable(self, offset, irsb_addr, stmt_id):
        tpl = (irsb_addr, stmt_id)
        if offset in self._stack_variable_map and \
                tpl in self._stack_variable_map[offset]:
            return self._stack_variable_map[offset][tpl]
        else:
            return None

    def get_stack_variables(self, offset):
        if offset in self._stack_variable_map and \
                len(self._stack_variable_map[offset]):
            return self._stack_variable_map[offset].values()
        else:
            return None

    @property
    def variables(self):
        return self._var_list

class VariableSeekr(object):
    def __init__(self, project, cfg, vfg):
        self._cfg = cfg
        self._vfg = vfg
        self._project = project

        # A shortcut to arch
        self._arch = project.arch

        self._variable_managers = {}

    def construct(self, func_start=None):
        self._do_work(func_start)

    def _variable_manager(self, function_start):
        if function_start not in self._variable_managers:
            variable_manager = VariableManager(function_start)
            self._variable_managers[function_start] = variable_manager

            return variable_manager
        else:
            return self._variable_managers[function_start]

    def _do_work(self, func_start):
        function_manager = self._cfg.function_manager
        functions = function_manager.functions

        if func_start not in functions:
            raise AngrInvalidArgumentError('No such function exists in FunctionManager: function 0x%x.', func_start)

        func = functions[func_start]

        var_idx = itertools.count()


        initial_run = self._vfg.get_any_irsb(func_start) # FIXME: This is buggy

        if initial_run is None:
            raise AngrInvalidArgumentError('No such SimRun exists in VFG: 0x%x.', func_start)

        run_stack = [initial_run]
        processed_runs = set()
        processed_runs.add(initial_run)

        while len(run_stack) > 0:
            current_run = run_stack.pop()

            if isinstance(current_run, SimIRSB):
                irsb = current_run

                memory = irsb.exits()[0].state.memory
                events = memory.state.log.events_of_type('uninitialized')
                print events
                for _, region in memory.regions.iteritems():
                    if region.is_stack:
                        for tpl, aloc in region.alocs.items():
                            irsb_addr, stmt_id = tpl
                            variable_manager = self._variable_manager(region.related_function_addr)
                            if variable_manager.get_stack_variable(aloc.offset, irsb_addr, stmt_id) is None:
                                stack_var = StackVariable(var_idx.next(), aloc.size, irsb_addr, stmt_id, 0, aloc.offset, 0)
                                variable_manager.add(stack_var)

            elif isinstance(current_run, SimProcedure):
                pass
                # simproc = current_run
                # for ref in simproc.refs():
                #     handler_name = '_handle_reference_%s' % type(ref).__name__
                #     if hasattr(self, handler_name):
                #         getattr(self, handler_name)(func, var_idx,
                #                                     variable_manager,
                #                                     regmap,
                #                                     temp_var_map,
                #                                     simproc,
                #                                     simproc.addr, -1,
                #                                     concrete_sp, ref)

            # Successors
            successors = self._vfg.get_successors(current_run, excluding_fakeret=False)
            for suc in successors:
                if suc not in processed_runs and suc.addr in func.basic_blocks:
                    run_stack.append(suc)
                    processed_runs.add(suc)

        # Post-processing
        if func.bp_on_stack:
            # base pointer is pushed on the stack. To be consistent with IDA,
            # we wanna adjust the offset of each stack variable
            for var in variable_manager.variables:
                if isinstance(var, StackVariable):
                    var.offset += self._arch.bits / 8

    def _collect_tmp_deps(self, tmp_tuple, temp_var_map):
        '''
        Return all registers that a tmp relies on
        '''
        reg_deps = set()
        for tmp_dep in tmp_tuple:
            if tmp_dep in temp_var_map:
                reg_deps |= temp_var_map[tmp_dep]
        return reg_deps

    def _try_concretize(self, se, exp):
        '''
        FIXME: This ia a temporary fix for simuvex Issue #31
        '''
        if type(exp) in [int, long]:
            return exp
        else:
            # Try to concretize it
            if se.symbolic(exp):
                return exp
            else:
                return se.any_int(exp)


    def _handle_reference_SimMemRead(self, func, var_idx, variable_manager, regmap, temp_var_map, current_run, ins_addr, stmt_id, concrete_sp, ref):
        addr = ref.addr
        if not addr.symbolic:
            concrete_addr = current_run.initial_state.se.any_int(addr)
            offset = concrete_addr - concrete_sp
            if abs(offset) < STACK_SIZE:
                # Let's see if this variable already exists
                existing_var = variable_manager.get_stack_variable(offset)
                if existing_var is not None:
                    # We found it!
                    # Add a reference to that variable
                    variable_manager.add_ref(existing_var, current_run.addr, stmt_id)
                else:
                    # This is a variable that is read before created/written
                    l.debug("Stack variable %d has never been written before.", offset)
                    size_ = self._try_concretize(current_run.initial_state.se, ref.size)
                    stack_var = StackVariable(var_idx.next(), size_, current_run.addr, stmt_id, ins_addr, offset, concrete_addr)
                    variable_manager.add(stack_var)
                    if offset > 0:
                        func.add_argument_stack_variable(offset)

    def _handle_reference_SimTmpWrite(self, func, var_idx, variable_manager, regmap, temp_var_map, current_run, ins_addr, stmt_id, concrete_sp, ref):
        tmp_var_id = ref.tmp
        temp_var_map[tmp_var_id] = set(ref.data_reg_deps)
        for tmp_dep in ref.data_tmp_deps:
            if tmp_dep in temp_var_map:
                temp_var_map[tmp_var_id] |= temp_var_map[tmp_dep]

    def _handle_reference_SimMemWrite(self, func, var_idx, variable_manager, regmap, temp_var_map, current_run, ins_addr, stmt_id, concrete_sp, ref):
        addr = ref.addr

        if not addr.symbolic:
            concrete_addr = current_run.initial_state.se.any_int(addr)
            offset = concrete_addr - concrete_sp
            if abs(offset) < STACK_SIZE:
                # What will be written?
                # If the value is the stack pointer, we don't treat it as a variable.
                # Instead, we'll report this fact to func.
                reg_deps = self._collect_tmp_deps(ref.data_tmp_deps, temp_var_map)
                reg_deps |= set(ref.data_reg_deps)
                if len(reg_deps) == 1 and self._arch.bp_offset in reg_deps:
                    # Report to func
                    func.bp_on_stack = True
                    return

                # As this is a write, it must be a new
                # variable (although it might be
                # overlapping with another variable).
                size_ = self._try_concretize(current_run.initial_state.se, ref.size)
                stack_var = StackVariable(var_idx.next(), size_, current_run.addr, stmt_id, ins_addr, offset, concrete_addr)
                variable_manager.add(stack_var)

    def _handle_reference_SimRegRead(self, func, var_idx, variable_manager, regmap, temp_var_map, current_run, ins_addr, stmt_id, concrete_sp, ref):
        if ref.offset in [self._arch.sp_offset, self._arch.ip_offset]:
            # Ignore stack pointer
            return
        if not regmap.contains(ref.offset):
            # The register has never been written before
            func.add_argument_register(ref.offset)

    def _handle_reference_SimRegWrite(self, func, var_idx, variable_manager, regmap, temp_var_map, current_run, ins_addr, stmt_id, concrete_sp, ref):
        if ref.offset in [self._arch.sp_offset, self._arch.ip_offset]:
            # Ignore stack pointers and program counter
            return
        regmap.assign(ref.offset, 1)

    def get_variable_manager(self, func_addr):
        if func_addr in self._variable_managers:
            return self._variable_managers[func_addr]
        else:
            return None

from .errors import AngrInvalidArgumentError



""" Angr module """
# pylint: disable=wildcard-import

import logging
logging.getLogger("angr").addHandler(logging.NullHandler())

from .project import *
from .functionmanager import *
from .variableseekr import *
from .regmap import *
from .mergeseekr import *
from .annocfg import *
from .path import *
from .errors import *
from .surveyor import *
from .service import *
from .analyses import *
from .analysis import *
from .tablespecs import *
from . import surveyors
from .blade import Blade
from .simos import SimOS
from .path_group import PathGroup
from .surveyors.caller import Callable



class RegisterMap(object):
    def __init__(self, arch):
        self._reg_map = {}
        # TODO: Properly set the width of a register
        self._general_register_width = arch.bytes

    def assign(self, reg_offset, expr):
        if reg_offset % self._general_register_width == 0:
            self._reg_map[reg_offset] = expr
        else:
            raise Exception("The offset %d is not aligned." % reg_offset)

    def contains(self, reg_offset):
        # TODO: Support unaligned offsets
        assert reg_offset % self._general_register_width == 0
        return reg_offset in self._reg_map

    def get(self, reg_offset):
        if reg_offset % self._general_register_width == 0:
            if reg_offset in self._reg_map:
                return self._reg_map[reg_offset]
            else:
                return None
        else:
            raise Exception("The offset is not aligned.")

    def remove(self, reg_offset):
        if reg_offset % self._general_register_width == 0:
            del self._reg_map[reg_offset]
        else:
            raise Exception("The offset is not aligned.")



from collections import defaultdict
import logging

import networkx

l = logging.getLogger("angr.analyses.cdg")

from ..analysis import Analysis

# Control dependency graph

class TempNode(object):
    def __init__(self, label):
        self._label = label

    def __repr__(self):
        return self._label

class CDG(Analysis):
    """
    Implements a control dependence graph.
    """

    def __init__(self, cfg=None, start=None, full_control_depencences=False):
        """
        Constructor.

        :param cfg: The control flow graph upon which this control dependence graph will build
        :param start: The starting point to begin constructing the control dependence graph
        :param full_control_depencences: Set to True to include all dependants of a node in the final graph, otherwise
                                        only those strict dominators are included. The former is constructed on
                                        the post dominance frontier mapping, while the latter is based on a post
                                        dominator tree.
        """
        self._project = self._p
        self._binary = self._project.loader.main_bin
        self._start = start
        self._full_control_dependences = full_control_depencences

        self._cfg = cfg if cfg is not None else self._p.analyses.CFG()
        self._acyclic_cfg = self._cfg.copy()
        # The CFG we use should be acyclic!
        self._acyclic_cfg.remove_cycles()

        self._ancestor = None
        self._semi = None
        self._post_dom = None

        self._graph = None
        self._label = None
        # Debugging purpose
        if hasattr(self._cfg, "get_node"):
            # FIXME: We should not use get_any_irsb in such a real setting...
            self._entry = self._cfg.get_any_node(self._p.entry)

        self._construct()

    #
    # Properties
    #

    @property
    def graph(self):
        return self._graph

    #
    # Public methods
    #

    def get_post_dominators(self):
        """
        Return the post-dom tree
        """
        return self._post_dom

    def get_dependants(self, run):
        """
        Return a list of nodes that are control dependent on the given node in the control dependence graph
        """
        if run in self._graph.nodes():
            return self._graph.predecessors(run)
        else:
            return []

    def get_guardians(self, run):
        """
        Return a list of nodes on whom the specific node is control dependent in the control dependence graph
        """
        if run in self._graph.nodes():
            return self._graph.successors(run)
        else:
            return []

    #
    # Private methods
    #

    def _construct(self):
        """
        Construct a control dependence graph.

        This implementtion is based on figure 6 of paper An Efficient Method of Computing Static Single Assignment
        Form by Ron Cytron, etc.
        """

        # Construct post-dominator tree
        self._pd_construct()

        self._graph = networkx.DiGraph()

        if self._full_control_dependences:
            # Construct the reversed dominance frontier mapping
            rdf = self._df_construct(self._post_dom)

            for y in self._cfg.graph.nodes_iter():
                if y not in rdf:
                    continue
                for x in rdf[y]:
                    self._graph.add_edge(x, y)

        else:
            for y in self._cfg.graph.nodes_iter():
                if y not in self._post_dom:
                    continue
                for x in self._post_dom.successors_iter(y):
                    self._graph.add_edge(x, y)

        self._post_process()

    def _post_process(self):
        """
        There are cases where a loop has two overlapping loop headers thanks
        to the way VEX is dealing with continuous instructions. As we were
        breaking the connection between the second loop header and its
        successor, we shall restore them in our CDG.
        """
        # TODO: Verify its correctness
        loop_back_edges = self._cfg.get_loop_back_edges()
        for b1, b2 in loop_back_edges:
            self._graph.add_edge(b1, b2)

    #
    # Dominance frontier related
    #

    def _df_construct(self, postdom):
        """
        Construct a dominance frontier based on the given post-dominator tree.

        This implementation is based on figure 2 of paper An Efficient Method of Computing Static Single Assignment
        Form by Ron Cytron, etc.

        :param postdom: The post-dominator tree
        :return: A dict of dominance frontier
        """

        DF = { }

        # Perform a post-order search on the post-dom tree
        for x in networkx.dfs_postorder_nodes(postdom):
            DF[x] = set()

            # local set
            for y in postdom.successors_iter(x):
                if postdom[y] is not x:
                    DF[x].add(y)

            # up set
            if x is None:
                continue

            for z in networkx.dfs_postorder_nodes(postdom, x):
                if z is x:
                    continue
                for y in DF[z]:
                    if postdom[y] is not x:
                        DF[x].add(y)

        return DF

    #
    # Post-dominator tree related
    #

    def _pd_construct(self):
        """
        Find post-dominators for each node in CFG.

        This implementation is based on paper A Fast Algorithm for Finding Dominators in a Flow Graph by Thomas
        Lengauer and Robert E. Tarjan from Stanford University, ACM Transactions on Programming Languages and Systems,
        Vol. 1, No. 1, July 1979
        """
        normalized_graph, vertices, parent = self._pd_normalize_graph()

        bucket = defaultdict(set)
        dom = [None] * (len(vertices))
        self._ancestor = [None] * (len(vertices) + 1)

        range_ = range(1, len(vertices))
        range_.reverse()
        for i in range_:
            w = vertices[i]
            if w not in parent:
                # It's one of the start nodes
                continue
            predecessors = normalized_graph.predecessors(w)
            for v in predecessors:
                u = self._pd_eval(v)
                if self._semi[u.index].index < self._semi[w.index].index:
                    self._semi[w.index] = self._semi[u.index]
            bucket[vertices[self._semi[w.index].index].index].add(w)
            self._pd_link(parent[w], w)
            for v in bucket[parent[w].index]:
                u = self._pd_eval(v)
                if self._semi[u.index].index < self._semi[v.index].index:
                    dom[v.index] = u
                else:
                    dom[v.index] = parent[w]
            bucket[parent[w].index].clear()

        for i in range(1, len(vertices)):
            w = vertices[i]
            if w not in parent:
                continue
            if dom[w.index] != vertices[self._semi[w.index].index]:
                dom[w.index] = dom[dom[w.index].index]

        self._post_dom = networkx.DiGraph() # The post-dom tree described in a directional graph
        for i in xrange(1, len(vertices)):
            self._post_dom.add_edge(vertices[i], dom[i])

        self._pd_post_process()

    def _pd_post_process(self):
        '''
        Take care of those loop headers/tails where we manually broke their
        connection to the next BBL
        '''
        loop_back_edges = self._cfg.get_loop_back_edges()
        for b1, b2 in loop_back_edges:
            # The edge between b1 and b2 is manually broken
            # The post dominator of b1 should be b2 (or not?)
            successors = self._acyclic_cfg.get_successors(b1)
            if len(successors) == 0:
                if b2 in self._post_dom:
                    self._post_dom.add_edge(b1, b2)
                else:
                    l.debug("%s is not in post dominator dict.", b2)

    def _pd_normalize_graph(self):
        # We want to reverse the CFG, and label each node according to its
        # order in a DFS
        graph = networkx.DiGraph()

        n = self._start if self._start is not None else self._entry
        assert n is not None
        queue = [n]
        start_node = TempNode("start_node")
        traversed_nodes = set()
        while len(queue) > 0:
            node = queue.pop()
            traversed_nodes.add(node)
            successors = self._acyclic_cfg.get_successors(node)
            if len(successors) == 0:
                # Add an edge between this node and our start node
                graph.add_edge(start_node, node)
            for s in successors:
                graph.add_edge(s, node) # Reversed
                if s not in traversed_nodes:
                    queue.append(s)

        # Add a start node and an end node
        graph.add_edge(n, TempNode("end_node"))

        all_nodes_count = len(traversed_nodes) + 2 # A start node and an end node
        l.debug("There should be %d nodes in all", all_nodes_count)
        counter = 0
        vertices = ["placeholder"]
        scanned_nodes = set()
        parent = {}
        while True:
            # DFS from the current start node
            stack = [start_node]
            while len(stack) > 0:
                node = stack.pop()
                counter += 1
                node.index = counter
                scanned_nodes.add(node)
                vertices.append(node)
                successors = graph.successors(node)
                for s in successors:
                    if s not in scanned_nodes:
                        stack.append(s)
                        parent[s] = node
                        scanned_nodes.add(s)

            if counter >= all_nodes_count:
                break

            l.debug("%d nodes are left out during the DFS. They must formed a cycle themselves.", all_nodes_count - counter)
            # Find those nodes
            leftovers = [s for s in traversed_nodes if s not in scanned_nodes]
            graph.add_edge(start_node, leftovers[0])
            # We have to start over...
            counter = 0
            parent = {}
            scanned_nodes = set()
            vertices = ["placeholder"]

        self._semi = vertices[::]
        self._label = vertices[::]

        return (graph, vertices, parent)

    def _pd_link(self, v, w):
        self._ancestor[w.index] = v

    def _pd_eval(self, v):
        if self._ancestor[v.index] is None:
            return v
        else:
            self._pd_compress(v)
            return self._label[v.index]

    def _pd_compress(self, v):
        if self._ancestor[self._ancestor[v.index].index] != None:
            self._pd_compress(self._ancestor[v.index])
            if self._semi[self._label[self._ancestor[v.index].index].index].index < self._semi[self._label[v.index].index].index:
                self._label[v.index] = self._label[self._ancestor[v.index].index]
            self._ancestor[v.index] = self._ancestor[self._ancestor[v.index].index]
